import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model
from PIL import Image
import matplotlib.image as mpimg
import os
from pathlib import Path


class Tanaka_Park():

    
    FC=27200
    REC=0.028        
    SUP_INF=4
    DER_INF=5
    E_ELASTIC=200000000

    #RECODERS
    FILE_DESPLAZAMIENTO = "DESPLAZAMIENTO_1.out"  # DESPLAZAMIENTO
    FILE_REAC_BASE = "REAC_BASE.out"            # REACCION_BASE
    FILE_INTER_ENS = "INTERSECCION_ENSA_FISI.out"  # INTERSECCION DE COMPARACION DE LOS ENSAYOS REALIZADOS

    def __init__(self,nombre='BasicBuilder',numero_dimensiones=2, numero_dots=3, data_file='Tanaka and Park 1990, Nº5.txt', altura_columna=2.335, tipo_concreto=1, tipo_refuerzo=2 ,dim_travs_colum_altura=0.55, dim_trav_colum_base=0.55, data_file_lab='Tanaka_Park.txt',  calculo='*'):
                
        self.nombre = nombre
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots                
        self.data_file = data_file
        self.tipo_concreto = tipo_concreto
        self.tipo_refuerzo = tipo_refuerzo

        #Datos de la Estructura
        self.altura_columna = altura_columna    
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.data_file_lab = data_file_lab
        self.data_file_lab = data_file_lab
        self.calculo = calculo
        
        ops.wipe()
        ops.model(self.nombre, "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

    def construir_modelo(self):
        
        fc=32000
        fyu_01=429000
        fyu=675000
        rec=0.04
        dia_estr=0.012
        Esp_estr=0.11
        nvsx=2
        nvsy=2
        dia_varcol=20 #int(input("Ingrese el diametro de la varilla de refuerzo para la columna: "))
        var_sup_inf=4
        var_der_izq=4
        areaFiber=((np.pi*dia_varcol**2)/4)/1000**2

        #Definir los nodos de la estructura
        ops.node(1, 0, 0)
        ops.node(2, 0, 1.65)

        #Restricciones    
        ops.fix(1, 1, 1, 1)

        # Definir los materiales para la columna
        # ------------------------------------------
        # CONCRET0  
        
        if self.tipo_concreto == 1:
            # Concreto no confinado
            fpc=-fc
            epsc0=-0.002
            fpcu=0.8*fc
            epsU=-0.004
            ops.uniaxialMaterial("Concrete01", 2 , -fpc , -epsc0 , -fpcu , -epsU)

            #Concreto Confinado
            esm=1
            Ec=(4700*(fc/1000)**0.5)*1000
            sp=Esp_estr-dia_estr
            rs=(nvsx*dia_estr**2*0.875*(self.dim_trav_colum_base-2*rec)+nvsy*dia_estr**2*0.875*(self.dim_travs_colum_altura-2*rec))/((self.dim_trav_colum_base-2*rec)*(self.dim_travs_colum_altura-2*rec)*Esp_estr)
            areaFiber=((np.pi*dia_varcol**2)/4)/1000**2
            area_conf=(self.dim_trav_colum_base-2*rec)*(self.dim_travs_colum_altura-2*rec)
            rcc=areaFiber*(2*var_sup_inf+2*var_der_izq)/area_conf
            wi=0.2
            ke=((1-wi**2/(6*area_conf))*(1-Esp_estr/(2*self.dim_trav_colum_base))*(1-Esp_estr/(2*self.dim_travs_colum_altura)))/(1-rcc)
            fpl=1/2*ke*rs*fyu_01
            fpcc=(-1.254+2.254*(1+7.94*fpl/fc)**0.5-2*fpl/fc)*fc
            ecc=epsc0*(1+5*(fpcc/fc-1))
            Esec=fpcc/ecc
            r=Ec/(Ec-Esec)
            ecu=1.5*(0.004+1.4*rs*fyu_01*esm/fpcc)
            fcu=0.8*fpcc
            ops.uniaxialMaterial("Concrete01", 1, -fpcc , -ecc , -fcu , -ecu)

        elif self.tipo_concreto == 2:
            # Concreto inconfinado
            fpc=-fc
            epsc0=-0.003
            fpcu=0.8*fc
            epsU=-0.006
            lambdaC=0.5
            ft=(0.62*(fc)**0.5)
            Ets=0.1*(2*(fc)/epsc0)

            ops.uniaxialMaterial("Concrete02", 2 , -fc , epsc0 , fpcu , epsU , lambdaC , ft , Ets)

            # Concreto confinado

            #Columnas

            fcc=1.35*fc
            epscc0=-0.003
            fpccu=0.8*fc
            epscU=-0.006

            ops.uniaxialMaterial("Concrete02", 1 , -fcc , epscc0 , fpccu , epscU , lambdaC , ft , Ets)
            
        elif self.tipo_concreto == 3:
            # Concreto inconfinado
            fpc=-fc
            epsc0=-0.003
            fpcu=0.8*fc
            epsU=-0.006
            endStrainSITC=0.03

            ops.uniaxialMaterial("Concrete01WithSITC", 2 , -fc , epsc0 , fpcu , epsU, endStrainSITC)

            # Concreto confinado

            #Columnas

            fcc=-1.35*fc
            epsc0=-0.003
            fpcu=0.8*fc
            epsU=-0.006
            endStrainSITC=0.03

            ops.uniaxialMaterial("Concrete01WithSITC", 1 , -fcc , epsc0 , fpcu , epsU , endStrainSITC)
            
        else:
            
            print("No se encontro el concreto a utilizar")

        #Material Elástico del Concreto
        E_elastic=20000000
        ops.uniaxialMaterial("Elastic", 3, E_elastic)

        #Acero de refuerzo
        
        if self.tipo_refuerzo==1:
            Fy=511000
            Fyu=fyu
            E0=200000000
            b=0.0179
            ops.uniaxialMaterial("Steel01", 4, Fy, E0, b)
            
        elif self.tipo_refuerzo==2:
            Fy=511000
            Fyu=fyu
            E0=200000000
            b=0.0179
            R0=15
            cR1=0.925
            cR2=0.15
            a1=0
            a2=1
            a3=0
            a4=1
            ops.uniaxialMaterial("Steel02", 4, Fy, E0, b, R0, cR1, cR2, a1, a2, a3, a4)

        else: 
            print("No se encontro el acero de refuerzo a utilizar")

        #Definir las etiquetas de secciones
        col_sec=1

        #Creacion de las secciones
        ops.section("Fiber", col_sec)

        y0=(self.dim_travs_colum_altura/2)-rec
        z0=(self.dim_trav_colum_base/2)-rec

        #Fibra del Concreto Confinado
        Eti_quad_conf=1
        numSubdivIJ=10
        numSubdivJK=10
        crdsI=[y0, z0]
        crdsJ=[y0, -z0]
        crdsK=[-y0, -z0]
        crdsL=[-y0, z0]
        ops.patch("quad", Eti_quad_conf, numSubdivIJ, numSubdivJK, *crdsI , *crdsJ , *crdsK , *crdsL)

        #Fibra del Concreto no Confinado
        y1=self.dim_travs_colum_altura/2
        z1=self.dim_trav_colum_base/2

        Eti_quad_noconf=2
        numSubdivIJ=1
        numSubdivJK=4
        crdsI=[y1, z1]
        crdsJ=[-y1, z1]
        crdsK=[y1, z0]
        crdsL=[-y1, z0]
        ops.patch("quad", Eti_quad_noconf, numSubdivIJ, numSubdivJK, * crdsI , * crdsJ , * crdsK , * crdsL)

        crdsI=[y1, -z1]
        crdsJ=[-y1, -z1]
        crdsK=[-y1, -z0]
        crdsL=[y1, -z0]
        ops.patch("quad", Eti_quad_noconf, numSubdivIJ, numSubdivJK, * crdsI , * crdsJ , * crdsK , * crdsL)

        crdsI=[y1, z0]
        crdsJ=[y1, -z0]
        crdsK=[y0, -z0]
        crdsL=[y0, z0]
        ops.patch("quad", Eti_quad_noconf, numSubdivIJ, numSubdivJK, * crdsI , * crdsJ , * crdsK , * crdsL)

        crdsI=[-y1, -z0]
        crdsJ=[-y1, z0]
        crdsK=[-y0, z0]
        crdsL=[-y0, -z0]
        ops.patch("quad", Eti_quad_noconf, numSubdivIJ, numSubdivJK, * crdsI , * crdsJ , * crdsK , * crdsL)

        #Definir el acero longitudinal
        matTag=4
        #Definir acero longitudinal superior
        numFiber=var_sup_inf
        start=[y0, z0]
        end=[y0, -z0]
        ops.layer("straight", matTag, numFiber, areaFiber, *start, *end)

        #Definir acero longitudinal inferior
        numFiber=var_sup_inf
        start=[-y0, z0]
        end=[-y0, -z0]
        ops.layer("straight", matTag, numFiber, areaFiber, *start, *end)

        #Definir acero longitudinal derecha
        numFiber=var_der_izq
        start=[y0, z0]
        end=[-y0, z0]
        ops.layer("straight", matTag, numFiber, areaFiber, *start, *end)

        #Definir acero longitudinal izquierda
        numFiber=var_der_izq
        start=[y0, -z0]
        end=[-y0, -z0]
        ops.layer("straight", matTag, numFiber, areaFiber, *start, *end)

        #Definir agregador de sección 
        Eti_Agr=2
        ops.section("Aggregator", Eti_Agr, 3, "Vy", "-section", col_sec)

        #Definir la transformacion de coordenadas
        eti_Transf=1
        ops.geomTransf("Linear", eti_Transf)

        #Definir los elementos de la columna
        Eti_beam=1
        N=2
        ops.beamIntegration("Lobatto", Eti_beam, Eti_Agr, N )
        eleTag=1
        eleNodes=[1, 2]
        ops.element("dispBeamColumn", eleTag, * eleNodes, eti_Transf, Eti_beam)

        #Definir el recorder
        nodo=2
        dof=1
        ops.recorder("Node", "-file", self.FILE_INTER_ENS, "-time", "-node", nodo, "-dof", dof, "disp")
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO, "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time", "-node", 1, "-dof", 1, 2, 3, "reaction")

        #Definir los patrones de carga
        ops.timeSeries("Constant", 1)
        ops.pattern("Plain", 1, 1)
        ops.load(nodo, 0.0, -968, 0.0)

        # Definir los parametros de analisis
        ops.integrator("LoadControl", 0)
        ops.system("SparseGeneral", "-piv")
        ops.test("NormDispIncr", 1.0e-4, 2000)
        ops.constraints("Plain")
        ops.numberer("Plain")
        ops.algorithm("KrylovNewton")
        ops.analysis("Static")

        ok=ops.analyze(1)
        if ok==0:
            print("CARGA AXIAL APLICADA")
        else:
            print("ERROR AL APLICAR LA CARGA AXIAL")
        ops.wipeAnalysis()

        if ok==0:
            
            # Definir patron de carga
            ops.timeSeries("Linear", 2)
            ops.pattern("Plain", 2, 2)
            ops.load(nodo, 1, 0.0, 0.0)
            
            #Definir los nodos de la estructura
            nodos=np.loadtxt(self.data_file)
            num_nodos=len(nodos)

            for i in range(len(nodos)):
                x=float(nodos[i])

                ops.integrator("DisplacementControl", nodo, dof, x)
                ops.analysis("Static")
                ops.analyze(1)

            ops.wipeAnalysis()
        ops.remove("recorders")
                            
         #Personalizar la carga de los datos
        
        serie_total= []        

        if self.calculo == 'REACCION_BASE' :                
            array_mek=np.loadtxt(self.FILE_REAC_BASE)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            


        elif self.calculo == 'DESPLAZAMIENTO_1':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)

        else:            
            datos_1=np.loadtxt(self.FILE_REAC_BASE)  
            datos_2=np.loadtxt(self.FILE_INTER_ENS)                                              
            datos_3=np.loadtxt(self.FILE_DESPLAZAMIENTO)

            serie_total = [[ datos_1, [1,2,3],'REACCION_BASE' ], [ datos_2, [1], 'INTERSECCION' ], [ datos_3, [1,2,3],'DESPLAZAMIENTO' ], ]
            #self.construye_figura(serie_total=serie_total, recoder='*')
        
        nodo=[1, 2]
        dof=[1, 2, 3]

        nodo1=nodo[0]
        nodo2=nodo[1]

        gra_liber1=dof[0]
        gra_liber2=dof[1]
        gra_liber3=dof[2]

        #Resultados

        RESULTADOS = []

        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber1)+" es: "+str(ops.nodeDisp(2, 1))+" m") )        
        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber2)+" es: "+str(ops.nodeDisp(2, 2))+" m") )        
        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber3)+" es: "+str(ops.nodeDisp(2, 3))+" m") )        

        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber1)+" es: "+str(ops.nodeReaction(1, 1))+" N" ))
        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber2)+" es: "+str(ops.nodeReaction(1, 2))+" N") )
        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber3)+" es: "+str(ops.nodeReaction(1, 3))+" N") )

        RESULTADOS.append(str("La reaccción en el elemento: "+ str(nodo1)+ " para el grado de libertad:"+ str(gra_liber1)+" es:"+str(ops.nodeReaction(1, 1))+" N"))

        if serie_total:
            self.construye_figura(serie_total=serie_total, recoder='*', resultados=RESULTADOS)

        self.mostrar_resultados(lista_mensajes=RESULTADOS) 


        #Este el el modelo realizado en el laboratorio                
        self.construye_figura([array_mek, array_mek_lab])


    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
        
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """
                
        titulo = None        
        
        if recoder != '*':

            ventana_principal = plt.figure(0)
            ventana_principal.canvas.set_window_title(recoder)

            for i in serie:
                plt.subplots_adjust(hspace=0.70)
                plt.subplot(2,2,i)
                titulo = "RESPUESTA : GRADO DE LIBERTAD #{}".format(i)            
                plt.title(titulo)                        
                plt.xlabel('Tiempo (seg)')
                plt.ylabel('Desplazamiento (m)')
                print ("Normal :",i)
                plt.plot(array_mek[:,0], array_mek[:,i],label="Resultado")
                plt.legend()                           
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent

            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'Carga_ciclica_01.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)


            num_subplots = 0
            
            for elemento in serie_total:
                num_subplots += len(elemento[1]) 
            
            if num_subplots % 2 != 0:
                num_subplots+=1
                        
            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 3
            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax4 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax5 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))
            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 2))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 2))
            
            #Para controlar el subplot de Resultados se visualice
            if COLUMNAS == 4: 
                ax9 = plt.subplot2grid((3,4), (0, 3), rowspan=3)
            

            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
            
                if calculo_var == 'REACCION_BASE':  
                    
                    ax3.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:orange')
                    ax3.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax3.set_xlabel('Fuerza (kN)')
                    ax3.set_ylabel('Desplazamiento (mm)')
                                        
                    ax4.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:orange')
                    ax4.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax4.set_xlabel('Fuerza (kN)')
                    ax4.set_ylabel('Desplazamiento (mm)')
                    
                    ax5.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:orange')
                    ax5.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax5.set_xlabel('Fuerza (kN)')
                    ax5.set_ylabel('Desplazamiento (mm)')

                elif calculo_var == 'INTERSECCION':
                                    
                    ax1.imshow(pil_img) 
                    ax1.set_title(f" ENSAYO POR CARGA CÍCLICA", fontsize=8)
                    ax1.axes.xaxis.set_visible(False)
                    ax1.axes.yaxis.set_visible(False)

                    #ax4.plot(array_mek[0][:,1], array_mek[0][:,0],'tab:orange')
                    ens_opensees=np.loadtxt(self.FILE_INTER_ENS)
                    ens_lab=np.loadtxt(self.data_file_lab)
                    ax2.plot(ens_opensees[:,1], ens_opensees[:,0],'tab:green')
                    ax2.plot(ens_lab[:,0], ens_lab[:,1],'tab:orange')
                    ax2.set_title(f" INTERSECCIÓN DE ENSAYOS", fontsize=10)
                    ax2.set_xlabel('Desplazamiento (m)')
                    ax2.set_ylabel('Fuerza (kN)')


                else:

                    ax6.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:red')
                    ax6.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Fuerza (kN)')
                    ax6.set_ylabel('Desplazamiento (m)')
                    
                    ax7.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:red')
                    ax7.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax7.set_xlabel('Fuerza (kN)')
                    ax7.set_ylabel('Desplazamiento (m)')

                    ax8.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:red')
                    ax8.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Fuerza (kN)')
                    ax8.set_ylabel('Desplazamiento (m)')                
                        
            if COLUMNAS == 4:
                self.mostrar_resultados(lista_mensajes=resultados,ax=ax9)

            fig.suptitle("MODELOS MATEMÁTICOS: ENSAYOS FISICOS")                             
            fig.subplots_adjust(left=0.05, bottom=0.076, right=0.971, top=0.88, wspace=0.562, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    


import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model, plot_modeshape

class CargaDistribuida():

    CARGA_DISTRIBUIDA = 150
    PES_ESP_HOR = 23.56
    GRAVEDAD = 9.81
    
    FILE_DESPLAZAMIENTO = "DESPLAZAMIENTO.out"          # DESPLAZAMIENTO N2
    FILE_REAC_BASE = "REAC_BASE.out"                    # REACCION_BASE 


    def __init__(self, nombre='BasicBuilder', numero_dimensiones=2, numero_dots=3, altura_columna=12, fuerza_aplicada=500, resistencia_comprension=21000 ,dim_travs_colum_altura=1.5, dim_trav_colum_base=1.5, tipo_concreto=1, acero_refuerzo=420000 , diametro_varilla_refuerzo = 20, varilla_columna_inferior = 3, varilla_columna_superior = 3):

        self.nombre = nombre
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots
        self.altura_columna = altura_columna
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.fuerza_aplicada = fuerza_aplicada
        self.resistencia_comprension = resistencia_comprension
        self.tipo_concreto = tipo_concreto
        self.acero_refuerzo = acero_refuerzo        
        self.diametro_varilla_refuerzo = diametro_varilla_refuerzo
        self.varilla_columna_inferior =  varilla_columna_inferior
        self.varilla_columna_superior = varilla_columna_superior


        ops.wipe()
        # Creación del modelo
        ops.model(self.nombre, "-ndm", self.numero_dimensiones,
                "-ndf", self.numero_dots)

    def construir_modelo(self):
        
        peso_ele = self.PES_ESP_HOR * self.dim_travs_colum_altura * \
            self.dim_trav_colum_base * self.altura_columna+self.CARGA_DISTRIBUIDA * self.altura_columna

        peso_estru = peso_ele+self.fuerza_aplicada
        
        masa = peso_estru/self.GRAVEDAD

        # Definir los materiales para la columna
        # ------------------------------------------
        # CONCRET0

        

        if self.tipo_concreto == 1:

            # Concreto inconfinado
            fpc = -self.resistencia_comprension
            epsc0 = -0.003
            fpcu = 0.2*self.resistencia_comprension
            epsu = -0.006

            ops.uniaxialMaterial("Concrete01", 1, -self.resistencia_comprension, epsc0, fpcu, epsu)

            # Concreto confinado

            # Columnas

            fcc = 1.35*fpc
            epscc0 = -0.008
            fpccu = 0.8*self.resistencia_comprension
            epsc_u = -0.020

            ops.uniaxialMaterial("Concrete01", 11, fcc, epscc0, fpccu, epsc_u)

        elif self.tipo_concreto == 2:
            # Concreto inconfinado
            fpc = -self.resistencia_comprension
            epsc0 = -0.003
            fpcu = 0.2*self.resistencia_comprension
            epsu = -0.006
            lambdac = 0.5
            ft = (0.62*(self.resistencia_comprension)**0.5)*1000
            ets = 0.1*(2*(self.resistencia_comprension)/epsc0)

            ops.uniaxialMaterial("Concrete02", 1, -self.resistencia_comprension, epsc0,
                                fpcu, epsu, lambdac, ft, ets)

            # Concreto confinado

            # Columnas

            fcc = 1.35*fpc
            epscc0 = -0.003
            fpccu = 0.2*self.resistencia_comprension
            epsc_u = -0.006

            ops.uniaxialMaterial("Concrete02", 11, fcc,
                                epscc0, fpccu, epsc_u, lambdac, ft, ets)

        elif self.tipo_concreto == 3:

            # Concreto inconfinado
            fpc = -self.resistencia_comprension
            epsc0 = -0.003
            fpcu = 0.2*self.resistencia_comprension
            epsu = -0.006
            end_strain_sitc = 0.03

            ops.uniaxialMaterial("Concrete01WithSITC",                                
                                1, -self.resistencia_comprension, epsc0, fpcu, epsu, end_strain_sitc)

            # Concreto confinado

            # Columnas

            fcc = -1.35*fpc
            epsc0 = -0.003
            fpcu = 0.2*self.resistencia_comprension
            epsu = -0.006
            end_strain_sitc = 0.03

            ops.uniaxialMaterial("Concrete01WithSITC", 11,
                                fcc, epsc0, fpcu, epsu, end_strain_sitc)

        elif self.tipo_concreto == 4:
            # Concreto inconfinado
            fpc = -self.resistencia_comprension
            epsc0 = -0.003
            fpcu = 0.2*self.resistencia_comprension
            ec_1 = (8500*(self.resistencia_comprension/1000)**0.5+110000)*1000
            beta = 0.1

            ops.uniaxialMaterial("Concrete04", 1, -self.resistencia_comprension, epsc0, fpcu, ec_1, beta)

            # Concreto confinado

            # Columnas

            fcc = 1.35*fpc
            epsc0 = -0.003
            fpcu = 0.2*self.resistencia_comprension
            ec_1 = (8500*(self.resistencia_comprension/1000)**0.5+110000)*1000
            beta = 0.1

            ops.uniaxialMaterial("Concrete04", 11, fcc, epsc0, fpcu,  ec_1, beta)

        elif self.tipo_concreto == 5:
            
            # Concreto inconfinado            
            e0 = -0.003
            n = 2
            k = 1
            alpha1 = 0.32
            ft = (0.62*(self.resistencia_comprension)**0.5)*1000
            ecr = 0.1*(2*(self.resistencia_comprension)/e0)
            b = 4
            alpha2 = 0.08

            ops.uniaxialMaterial("Concrete05", 1, -self.resistencia_comprension, e0,
                                n, k, alpha1, ft, ecr, b, alpha2)

            # Concreto confinado
            fcc = -1.35*self.resistencia_comprension
            e0 = -0.003
            n = 2
            k = 1
            alpha1 = 0.32
            ft = (0.62*(self.resistencia_comprension)**0.5)*1000
            ecr = 0.1*(2*(self.resistencia_comprension)/e0)
            b = 4
            alpha2 = 0.08

            ops.uniaxialMaterial("Concrete05", 11, -fcc,
                                e0, n, k, alpha1, ft, ecr, b, alpha2)

        elif self.tipo_concreto == 6:
            # Concreto inconfinado            
            ec = (((self.resistencia_comprension/1000)**0.25)/28)*1000
            ec_1 = (8200*(self.resistencia_comprension/1000)**(3/8))*1000
            ft = (0.62*(self.resistencia_comprension)**0.5)*1000
            et = (2*ft)/ec_1
            xp = 2
            xn = 2.3
            r = (((self.resistencia_comprension/1000)/5.2)-1.9)*1000

            ops.uniaxialMaterial("Concrete06", 1, -self.resistencia_comprension, ec,
                                ec_1, ft, et, xp, xn, r)

            # Concreto confinado
            fcc = -1.35*self.resistencia_comprension
            ec = (((self.resistencia_comprension/1000)**0.25)/28)*1000
            ec_1 = (8200*(self.resistencia_comprension/1000)**(3/8))*1000
            ft = (0.62*(self.resistencia_comprension)**0.5)*1000
            et = (2*ft)/ec_1
            xp = 2
            xn = 2.3
            r = (((self.resistencia_comprension/1000)/5.2)-1.9)*1000

            ops.uniaxialMaterial("Concrete06", 11, -fcc,
                                ec, ec_1, ft, et, xp, xn, r)
        else:
            print("No se encontro el concreto a utilizar")

        # ACERO DE REFUERZO                
        E0 = 200000000
        b = 0.007

        ops.uniaxialMaterial('Steel02', 2, self.acero_refuerzo, E0, b)

        # COLUMNAS
        # Definir la sección a la que se va a utilizar

        GJ = 1e14
        ops.section('Fiber', 2, '-GJ', GJ)

        # Definir las dimensiones

        B = self.dim_trav_colum_base
        H = self.dim_travs_colum_altura        
        cover = 0.04

        # Seccion de concreto
        self.concrete_section(11, 1, B, H, cover, 20, 5)

        # Definir el acero en la columna
        
        area_fiber = ((np.pi*self.diametro_varilla_refuerzo**2)/4)/1000**2

        # Aero inferior
                
        inicio = [-H/2+cover, -B/2+cover]
        fin = [-H/2+cover, B/2-cover]

        ops.layer('straight', 2, self.varilla_columna_inferior, area_fiber, * inicio, * fin)

        # Acero superior                
        inicio = [H/2-cover, -B/2+cover]
        fin = [H/2-cover, B/2-cover]

        ops.layer('straight', 2, self.varilla_columna_superior, area_fiber, * inicio, * fin)

        # Acero derecha
        # int(input("Ingrese el numero de varilla de la columna parte derecha: "))
        num_der = 3
        inicio = [H/2-0.20, -B/2+cover]
        fin = [H/2+0.2, B/2-cover]

        ops.layer('straight', 2, num_der, area_fiber, * inicio, * fin)

        # Acero izquierda
        # int(input("Ingrese el numero de varilla de la columna parte izquierda: "))
        num_izq = 3
        inicio = [H/2-0.20, -B/2-cover]
        fin = [H/2+0.20, B/2-cover]

        ops.layer('straight', 2, num_izq, area_fiber, * inicio, * fin)

        # Definir los nodos de la estructura

        ops.node(1, 0, 0)
        ops.node(2, 0, self.altura_columna)

        # Restricciones
        ops.fixY(0, 1, 1, 1)

        # Definir la transformacion de coordenadas
        ops.geomTransf("Linear", 1)

        # Definir los elementos de la estructura

        ops.beamIntegration(
            'HingeRadau', 2, 2, self.dim_travs_colum_altura/2, 2, self.dim_travs_colum_altura/2, 2)
        ops.element('forceBeamColumn', 1, 1, 2, 1, 2)

        # Definir los recorders
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO,
                    "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time",
                    "-node", 1, "-dof", 1, 2, 3, "reaction")
        
        # Definir los patrones de carga

        ops.timeSeries("Constant", 1)
        ops.pattern("Plain", 1, 1)

        # Cargas

        ops.load(2, 0, (-1*masa*self.GRAVEDAD), 0)

        ops.mass(2, (masa/self.GRAVEDAD+8), 1e-8, 1e-8)

        # Crear el analisis

        ops.constraints("Plain")
        ops.numberer("Plain")
        ops.system("BandGeneral")
        ops.test("EnergyIncr", 1.0e-6, 10)
        ops.algorithm("Newton")
        numero_pasos = 10
        ops.integrator("LoadControl", 1/numero_pasos)
        ops.analysis("Static")
        ok = ops.analyze(numero_pasos)

        if ok == 0:
            print("CARGA AXIAL APLICADA")
        else:
            print("ERROR AL APLICAR LA CARGA AXIAL")

        ops.loadConst("-time", 0.0)

        # Analisis modal
        num_modos = 1
        omega_sq = ops.eigen("-fullGenLapack", num_modos)
        omega_sq = np.array(omega_sq)
        omega = omega_sq**0.5
        periodo = (2*np.pi)/omega

        for i in range(num_modos):
            print("Modo", i+1, "T= ", periodo[i], "s")

        ops.remove("recorders")

        nodo = [1, 2]
        dof = [1, 2, 3]

        nodo1 = nodo[0]
        nodo2 = nodo[1]

        gra_liber1 = dof[0]
        gra_liber2 = dof[1]
        gra_liber3 = dof[2]

        # Resultados
        RESULTADOS = []

        RESULTADOS.append(str("El desplazamiento en el nodo ")+ str(nodo2) +" para el grado de libertad "+str(gra_liber1)+" es:"+str(ops.nodeDisp(2, 1))+" m")
        RESULTADOS.append(str("El desplazamiento en el nodo ")+ str(nodo2) +" para el grado de libertad "+str(gra_liber2)+" es:"+str(ops.nodeDisp(2, 2))+" m")
        RESULTADOS.append(str("El desplazamiento en el nodo ")+ str(nodo2) +" para el grado de libertad "+str(gra_liber3)+" es:"+str(ops.nodeDisp(2, 3))+" rad/s")

        RESULTADOS.append(str("La reaccción en el nodo ")+ str(nodo1) +" para el grado de libertad "+str(gra_liber1)+" es:"+str(ops.nodeReaction(1, 1))+" kN")
        RESULTADOS.append(str("La reaccción en el nodo ")+ str(nodo1) +" para el grado de libertad "+str(gra_liber2)+" es:"+str(ops.nodeReaction(1, 2))+" kN")
        RESULTADOS.append(str("La reaccción en el nodo ")+ str(nodo1) +" para el grado de libertad "+str(gra_liber3)+" es:"+str(ops.nodeReaction(1, 3))+" rad/s")
                    
        datos_1=np.loadtxt(self.FILE_DESPLAZAMIENTO)        
        datos_2=np.loadtxt(self.FILE_REAC_BASE)        
                
        serie_total = [ [ datos_1, [1,2,3],'DESPLAZAMIENTO' ], [ datos_2, [1,2,3],'REACCION_BASE' ] ]

        if serie_total:
            self.construye_figura(serie_total=serie_total, recoder='*')

        self.mostrar_resultados(lista_mensajes=RESULTADOS)


        
        
    def concrete_section(self, core_tag, cover_tag, B, H, cover, nfcore, nfcover):
        ops.patch("rect", core_tag, nfcore, 1, -H/2 +
                cover, -B/2+cover, H/2+cover, B/2+cover)
        ops.patch("rect", cover_tag, nfcover,
                1, -H/2, -B/2, -H/2+cover, B/2)
        ops.patch("rect", cover_tag, nfcover, 1, H/2-cover, -B/2, H/2, B/2)
        ops.patch("rect", cover_tag, nfcore, 1, -H /
                2+cover, -B/2, H/2-cover, B/2+cover)
        ops.patch("rect", cover_tag, nfcore, 1, -H /
                2+cover, B/2-cover, H/2-cover, B/2)


    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
        
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """
                
        titulo = None        
        
        if recoder != '*':

            ventana_principal = plt.figure(0)
            ventana_principal.canvas.set_window_title(recoder)

            for i in serie:
                plt.subplots_adjust(hspace=0.70)
                plt.subplot(2,2,i)
                titulo = "RESPUESTA : GRADO DE LIBERTAD #{}".format(i)            
                plt.title(titulo)                        
                plt.xlabel('Desplazamiento H. (mm)')
                plt.ylabel('Carga Aplicada (Kn)')
                print ("Normal :",i)
                plt.plot(array_mek[:,0], array_mek[:,i],label="Resultado")
                plt.legend()                           
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent

            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'Columna_con_carga_distribuida.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)

            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 3
            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax1.imshow(pil_img) 
            ax1.axes.xaxis.set_visible(False)
            ax1.axes.yaxis.set_visible(False)

            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax2.axes.xaxis.set_visible(False)
            ax2.axes.yaxis.set_visible(False)
            ax2.spines['top'].set_visible(False)
            ax2.spines['right'].set_visible(False)
            ax2.spines['bottom'].set_visible(False)
            ax2.spines['left'].set_visible(False) 

            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax4 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax5 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))
            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 2))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 2))
            
            #Para controlar el subplot de Resultados se visualice
            if COLUMNAS == 4: 
                ax9 = plt.subplot2grid((3,4), (0, 3), rowspan=3)
            

            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
                                        
                if calculo_var == 'REACCION_BASE':  
                    
                    ax3.plot(array_mek[:,0], array_mek[:,0],'tab:orange')
                    ax3.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax3.set_xlabel('Desplazamiento H. (mm)')
                    ax3.set_ylabel('Carga Aplicada (Kn)')
                                        
                    ax4.plot(array_mek[:,0], array_mek[:,1],'tab:orange')
                    ax4.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax4.set_xlabel('Desplazamiento H. (mm)')
                    ax4.set_ylabel('Carga Aplicada (Kn)')
                    
                    ax5.plot(array_mek[:,0], array_mek[:,2],'tab:orange')
                    ax5.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax5.set_xlabel('Desplazamiento H. (mm)')
                    ax5.set_ylabel('Carga Aplicada (Kn)')

                elif calculo_var == 'DESPLAZAMIENTO':

                    ax6.plot(array_mek[:,0], array_mek[:,0],'tab:red')
                    ax6.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Desplazamiento H. (mm)')
                    ax6.set_ylabel('Carga Aplicada (Kn)')
                    
                    ax7.plot(array_mek[:,0], array_mek[:,1],'tab:red')
                    ax7.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}\n", fontsize=8)
                    ax7.set_xlabel('Desplazamiento H. (mm)')
                    ax7.set_ylabel('Carga Aplicada (Kn)')

                    ax8.plot(array_mek[:,0], array_mek[:,2],'tab:red')
                    ax8.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Desplazamiento H. (mm)')
                    ax8.set_ylabel('Carga Aplicada (Kn)')
                    
                else:            
                    pass
            
            fig.suptitle("MODELOS MATEMÁTICOS: 4.2	COLUMNA DE HORMIGÓN ARMADO CON CARGA DISTRIBUIDA A LO LARGO DE LA COLUMNA")                             
            fig.subplots_adjust(left=0.05, bottom=0.076, right=0.850, top=0.88, wspace=0.562, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    def mostrar_resultados(self, lista_mensajes=[],ax=None):
                
        fig, ax1 = plt.subplots(1, 1, constrained_layout=True, sharey=True)       
        ax1.set_xlabel('Desplazamiento H. (mm)')
        ax1.set_ylabel('Carga Aplicada (Kn)')   
                        
        resultado = ''
        
        for index, cadena in enumerate(lista_mensajes):           
            resultado += cadena+'\n'
                        
        ax1.axes.xaxis.set_visible(False)
        ax1.axes.yaxis.set_visible(False)
        ax1.spines['top'].set_visible(False)
        ax1.spines['right'].set_visible(False)
        ax1.spines['bottom'].set_visible(False)
        ax1.spines['left'].set_visible(False)
        ax1.text(0.038, 0.75, resultado , horizontalalignment='left', verticalalignment='center_baseline', transform=ax1.transAxes, bbox={'facecolor': 'orange', 'alpha': 0.25, 'pad': 5}, fontsize=8)        
        fig.suptitle('\n\nResultados', fontsize=16)

        plt.show()    



import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt


class Momento_Curvatura():

    #RECODERS
    FILE_MOMENTO_CURVATURA = "Momento_curvatura.out"  # MOMENTO CURVATURA
    FILE_MATERIAL_1_TOP = "Material1+Top.out"  # MATERIAL 1 TOP
    FILE_MATERIAL_2_TOP = "Material2+Top.out"  # MATERIAL 2 TOP
    
    FILE_MATERIAL_1_BOT = "Material1+Bot.out"  # MATERIAL 1 BOT
    FILE_MATERIAL_2_BOT = "Material2+Bot.out"  # MATERIAL 2 BOT

    
    def __init__(self,nombre='BasicBuilder',numero_dimensiones=2, numero_dots=3, altura_columna=1.842, dim_travs_colum_altura=0.50, dim_trav_colum_base=0.50, calculo='*'):
                
        self.nombre = nombre
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots                

        #Datos de la Estructura
        self.altura_columna = altura_columna        
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.calculo = calculo
        
        ops.wipe()
        ops.model("basic", "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

        
    def construir_modelo(self):

        #Datos
        self.dim_travs_colum_altura=0.50 #float(input("Ingrese las dimensiones tranversal de la columna, altura: "))
        self.dim_trav_colum_base=0.50 #loat(input("Ingrese las dimensiones tranversal de la columna, base: "))
        #h=1.842 #t(input("Ingrese la altura de la columna: "))

        fc=28000
        rec=0.04
        dia_varcol=16        
        var_sup_inf=4
        var_der_izq=4
        areaFiber=((np.pi*dia_varcol**2)/4)/1000**2

        # Definir los materiales para la columna
        # ------------------------------------------
        # CONCRET0                   
        # Concreto confinado
        fpc=-fc
        epsc0=-0.003
        fpcu=0.2*fpc
        epsU=-0.006
        lambdaC=0.5
        ft=(0.62*(28)**0.5)*1000
        Ets=(2*(fpc)/epsc0)

        ops.uniaxialMaterial("Concrete02", 1 , fpc , epsc0 , fpcu , epsU , lambdaC , ft , Ets)

        # ACERO DE REFUERZO
        # Acero de refuerzo
        Fy=420000
        E0=200000000
        b=0.007

        ops.uniaxialMaterial( 'Steel02' , 2 , Fy , E0 , b)

        #Definir la sección a la que se va a utilizar
        GJ=1e14
        ops.section( 'Fiber' , 1 , '-GJ' , GJ )

        numSubdivY=20
        numSubdivZ=1
        B=0.40
        H=0.40
        crdsI=[-H/2, -B/2]
        crdsJ=[H/2, B/2]

        ops.patch( 'rect' , 1 , numSubdivY , numSubdivZ , * crdsI , * crdsJ )

        #Definir el acero en la viga
        numFiberSup=var_sup_inf
        areaFiber6=284e-6
        inicio=[-H/2+0.05, -B/2+0.05]
        fin=[-H/2+0.05, B/2-0.05]

        ops.layer( 'straight' , 2 , numFiberSup , areaFiber , * inicio , * fin )

        numFiberInf=var_sup_inf
        areaFiber7=387e-6
        inicio=[H/2-0.05, -B/2+0.05]
        fin=[H/2-0.05, B/2-0.05]

        ops.layer( 'straight' , 2 , numFiberInf , areaFiber , * inicio , * fin )

        numFiberSup=var_der_izq
        areaFiber6=284e-6
        inicio=[H/2+0.05, B/2+0.05]
        fin=[-H/2+0.05, B/2-0.05]

        ops.layer( 'straight' , 2 , numFiberSup , areaFiber , * inicio , * fin )

        numFiberInf=var_der_izq
        areaFiber7=387e-6
        inicio=[H/2-0.05, -B/2+0.05]
        fin=[-H/2-0.05, -B/2-0.05]

        ops.layer( 'straight' , 2 , numFiberInf , areaFiber , * inicio , * fin )

        # Analisis de momento-curvatura
        # Carga axial en la sección
        P= 750;

        # Nodos
        ops.node(1, 0.0, 0.0)
        ops.node(2, 0.0, 0.0)

        # Restriccuiones
        ops.fix(1, 1, 1, 1)
        ops.fix(2, 0, 1, 0)

        # Elemento de longitud cero
        ops.element("zeroLengthSection", 1, 1, 2, 1)

        # Patron de carga axial
        ops.timeSeries("Linear", 1)
        ops.pattern("Plain", 1, 1)
        ops.load(2, -P, 0.0, 0.0)

        #Crear el analisis
        ops.constraints("Plain")
        ops.numberer("Plain")
        ops.system("BandGeneral")
        ops.test("EnergyIncr", 1.0e-6, 30)
        ops.algorithm("Newton")
        NSteps=10
        ops.integrator("LoadControl", 1/NSteps)
        ops.analysis("Static")

        ok=ops.analyze(NSteps)
        if ok==0:
            print("CARGA AXIAL APLICADA")
        else:
            print("ERROR AL APLICAR LA CARGA AXIAL")
        ops.wipeAnalysis()

        ops.loadConst("-time", 0.0)

        if ok==0:
            # Definir patron de carga
            ops.timeSeries("Linear", 2)
            ops.pattern("Plain", 2, 2)
            ops.load(2, 0.0, 0.0, 1.0)
            
            #Crear el analisis
            ops.constraints("Plain")
            ops.numberer("Plain")
            ops.system("BandGeneral")
            ops.test("EnergyIncr", 1.0e-6, 1000)
            ops.algorithm("Newton")
            maxk=0.0021/(0.7*H)*20
            numIncr=1000
            ops.integrator("DisplacementControl", 2, 3, maxk/numIncr)
            ops.analysis("Static")
            
            # Recorder almacena Momento-Deformacion
            ops.recorder("Node", "-file", self.FILE_MOMENTO_CURVATURA, "-time", "-node", 2,
                        "-dof", 1, 3, "disp")
            
            # Recorder almacena Momento-Deformacion
            ops.recorder("Element", "-file", self.FILE_MATERIAL_1_TOP, "-time","-ele", 
                        1, "section", "fiber", "1.0", "0.0", "1", "stressStrain" )
            
            ops.recorder("Element", "-file", self.FILE_MATERIAL_2_TOP, "-time","-ele", 
                        1, "section", "fiber", "1.0", "0.0", "2", "stressStrain" )
            
            ops.recorder("Element", "-file", self.FILE_MATERIAL_1_BOT, "-time","-ele", 
                        1, "section", "fiber", "-1.0", "0.0", "1", "stressStrain" )
            
            ops.recorder("Element", "-file", self.FILE_MATERIAL_2_BOT, "-time","-ele", 
                        1, "section", "fiber", "-1.0", "0.0", "2", "stressStrain" )
            
            while ops.nodeDisp(2, 3) <= maxk:
                ok=ops.analyze(1)
                if ok != 0:
                    print("EL ANALISIS DE MOMENTO CURVATURA FALLO: ", ops.nodeDisp(2, 3))
                    break
            if ok == 0:
                print("EL ANALISIS TERMINO EXITOSAMENTE")

            ops.wipeAnalysis()
            ops.remove("recorders")
            
        datos_1=np.loadtxt(self.FILE_MOMENTO_CURVATURA)  

        serie_total = [ [ datos_1, [1,2,3],'MOMENTO_CURVATURA'] ]

        if serie_total:
            self.construye_figura(serie_total=serie_total, recoder='*')

    
    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
        
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """
                
        titulo = None        
        
        if recoder != '*':

            ventana_principal = plt.figure(0)
            ventana_principal.canvas.set_window_title(recoder)

            for i in serie:
                plt.subplots_adjust(hspace=0.70)
                plt.subplot(2,2,i)
                titulo = "RESPUESTA : GRADO DE LIBERTAD #{}".format(i)            
                plt.title(titulo)                        
                plt.xlabel('Desplazamiento H. (m)')
                plt.ylabel('Fuerza Cortante (Kn)')
                print ("Normal :",i)
                plt.plot(array_mek[:,0], array_mek[:,i],label="Resultado")
                plt.legend()                           
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent
            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'columna.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)

            IMAGEN_COLUMNA_ENSAYOS = os.path.join(os.path.join(BASE_DIR,'modelos'), 'labEnsayo1.jpg')
            pil_img_ensayos = Image.open(IMAGEN_COLUMNA_ENSAYOS)
            

            num_subplots = 0
            
            for elemento in serie_total:
                num_subplots += len(elemento[1]) 
            
            if num_subplots % 2 != 0:
                num_subplots+=1
                        
            fig = plt.figure()


            FILAS = 2
            COLUMNAS = 3

            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax1.imshow(pil_img) 
            ax1.axes.xaxis.set_visible(False)
            ax1.axes.yaxis.set_visible(False)

            """ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax2.axes.xaxis.set_visible(False)
            ax2.axes.yaxis.set_visible(False)
            ax2.spines['top'].set_visible(False)
            ax2.spines['right'].set_visible(False)
            ax2.spines['bottom'].set_visible(False)
            ax2.spines['left'].set_visible(False)"""

            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1), rowspan=2)

            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2), rowspan=2)
            ax6.imshow(pil_img_ensayos) 
            ax6.axes.xaxis.set_visible(False)
            ax6.axes.yaxis.set_visible(False)

    
            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
            
                if calculo_var == 'MOMENTO_CURVATURA':
                    ax3.plot(array_mek[:,2], array_mek[:,0],'tab:orange')
                    ax3.set_title(f" {calculo_var}", fontsize=8)
                    ax3.set_xlabel('Desplazamiento H. (m)')
                    ax3.set_ylabel('Fuerza Cortante (Kn)')
                else:
                    pass
            
            fig.suptitle("ENSAYOS MATEMÁTICOS: MOMENTO CURVATURA")                             
            fig.subplots_adjust(left=0.05, bottom=0.076, right=0.971, top=0.88, wspace=0.562, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("ENSAYOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    def mostrar_resultados(self, lista_mensajes=[],ax=None):
                
        fig, ax1 = plt.subplots(1, 1, constrained_layout=True, sharey=True)       
        ax1.set_xlabel('Desplazamiento H. (m)')
        ax1.set_ylabel('Fuerza Cortante (Kn)')   
                        
        resultado = ''
        
        for index, cadena in enumerate(lista_mensajes):           
            resultado += cadena+'\n'
                        
        ax1.axes.xaxis.set_visible(False)
        ax1.axes.yaxis.set_visible(False)
        ax1.spines['top'].set_visible(False)
        ax1.spines['right'].set_visible(False)
        ax1.spines['bottom'].set_visible(False)
        ax1.spines['left'].set_visible(False)
        ax1.text(0.038, 0.75, resultado , horizontalalignment='left', verticalalignment='center_baseline', transform=ax1.transAxes, bbox={'facecolor': 'orange', 'alpha': 0.25, 'pad': 5}, fontsize=9)        
        fig.suptitle('\n\nResultados', fontsize=16)

        plt.show()



import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model
from PIL import Image
import matplotlib.image as mpimg
import os
from pathlib import Path


class Wehbe():

    
    FC=27200
    REC=0.028        
    SUP_INF=4
    DER_INF=5
    E_ELASTIC=200000000

    #RECODERS
    FILE_DESPLAZAMIENTO = "DESPLAZAMIENTO_1.out"  # DESPLAZAMIENTO
    FILE_REAC_BASE = "REAC_BASE.out"            # REACCION_BASE
    FILE_INTER_ENS = "INTERSECCION_ENSA_FISI.out"  # INTERSECCION DE COMPARACION DE LOS ENSAYOS REALIZADOS

    def __init__(self,nombre='BasicBuilder',numero_dimensiones=2, numero_dots=3, data_file='Wehbe et al. 1998, A1.txt', altura_columna=2.335, diametro_columna=19.1, tipo_concreto=1, tipo_refuerzo=2 ,dim_travs_colum_altura=0.61, dim_trav_colum_base=0.38, data_file_lab='Whebe.txt', calculo='*'):
                
        self.nombre = nombre
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots                
        self.data_file = data_file
        self.tipo_concreto = tipo_concreto
        self.tipo_refuerzo = tipo_refuerzo

        #Datos de la Estructura
        self.altura_columna = altura_columna
        self.diametro_columna = diametro_columna
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.data_file_lab = data_file_lab
        self.calculo = calculo
        
        ops.wipe()
        ops.model(self.nombre, "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

    def construir_modelo(self):
                                                
        
        area_fiber=((np.pi*self.diametro_columna**2)/4)/1000**2

        #Definir los nodos de la estructura
        ops.node(1, 0, 0)
        ops.node(2, 0.58375, 0)
        ops.node(3, 1.1675, 0)
        ops.node(4, 1.75125, 0)
        ops.node(5, 2.335, 0)

        #Restricciones    
        ops.fix(1, 1, 1, 1)

        # Definir los materiales para la columna
        # ------------------------------------------
        # CONCRET0  
        
        if self.tipo_concreto == 1:
            # Concreto no confinado
            fpc=-self.FC
            epsc0=-0.002
            fpcu=0.8*self.FC
            epsU=-0.006
            ops.uniaxialMaterial("Concrete01", 2 , -fpc , -epsc0 , -fpcu , -epsU)

            #Concreto Confinado
            fpcc=self.FC+5521.13
            ecc=4.02982911046149E-03
            fcu=0.8*fpcc
            ecu=1.13249292446053E-02
            ops.uniaxialMaterial("Concrete01", 1, -fpcc , -ecc , -fcu , -ecu)

        elif self.tipo_concreto == 2:
            # Concreto inconfinado
            #fpc=-self.FC
            epsc0=-0.002
            fpcu=0.8*self.FC
            epsU=-0.006
            lambdac=0.5
            ft=(0.62*(self.FC)**0.5)
            ets=0.1*(2*(self.FC)/epsc0)

            ops.uniaxialMaterial("Concrete02", 2 , -self.FC , -epsc0 , -fpcu , -epsU , lambdac , ft , ets)

            # Concreto confinado

            #Columnas

            fcc=self.FC+5521.13
            epscc0=4.02982911046149E-03
            fpccu=0.8*self.FC
            epscu=1.13249292446053E-02

            ops.uniaxialMaterial("Concrete02", 1 , -fcc , -epscc0 , -fpccu , -epscu , lambdac , ft , ets)
            
        else:
            
            print("No se encontro el concreto a utilizar")

        #Material Elástico del Concreto
        
        ops.uniaxialMaterial("Elastic", 3, self.E_ELASTIC)

        #Acero de refuerzo
        
        if self.tipo_refuerzo==1:
            FY=448000
            E0=200000000
            b=1.44742225859247E-02
            a1=0.04
            a2=1
            a3=0.04
            a4=1
            ops.uniaxialMaterial("Steel01", 4, FY, E0, b, a1, a2, a3, a4)
            
        elif self.tipo_refuerzo==2:
            FY=448000
            E0=200000000
            b=1.44742225859247E-02
            R0=15
            CR1=0.925
            CR2=0.15
            a1=0.04
            a2=1
            a3=0.04
            a4=1
            ops.uniaxialMaterial("Steel02", 4, FY, E0, b, R0, CR1, CR2, a1, a2, a3, a4)

        else: 
            print("No se encontro el acero de refuerzo a utilizar")

        #Definir las etiquetas de secciones
        col_sec=1

        #Creacion de las secciones
        ops.section("Fiber", col_sec)

        y0=(self.dim_travs_colum_altura/2)-self.REC
        z0=(self.dim_trav_colum_base/2)-self.REC

        #Fibra del Concreto Confinado
        eti_quad_conf=1
        num_sub_div_ij=10
        num_sub_div_jk=10
        crds_i=[y0, z0]
        crds_j=[y0, -z0]
        crds_k=[-y0, -z0]
        crds_l=[-y0, z0]
        ops.patch("quad", eti_quad_conf, num_sub_div_ij, num_sub_div_jk, *crds_i , *crds_j , *crds_k , *crds_l)

        #Fibra del Concreto no Confinado
        y1=self.dim_travs_colum_altura/2
        z1=self.dim_trav_colum_base/2

        eti_quad_no_conf=2
        num_sub_div_ij=1
        num_sub_div_jk=4
        crds_i=[y1, z1]
        crds_j=[-y1, z1]
        crds_k=[y1, z0]
        crds_l=[-y1, z0]
        ops.patch("quad", eti_quad_no_conf, num_sub_div_ij, num_sub_div_jk, * crds_i , * crds_j , * crds_k , * crds_l)

        num_sub_div_ij=1
        num_sub_div_jk=4
        crds_i=[y1, -z1]
        crds_j=[-y1, -z1]
        crds_k=[-y1, -z0]
        crds_l=[y1, -z0]
        ops.patch("quad", eti_quad_no_conf, num_sub_div_ij, num_sub_div_jk, * crds_i , * crds_j , * crds_k , * crds_l)

        num_sub_div_ij=4
        num_sub_div_jk=1
        crds_i=[y1, z0]
        crds_j=[y1, -z0]
        crds_k=[y0, -z0]
        crds_l=[y0, z0]
        ops.patch("quad", eti_quad_no_conf, num_sub_div_ij, num_sub_div_jk, * crds_i , * crds_j , * crds_k , * crds_l)

        num_sub_div_ij=4
        num_sub_div_jk=1
        crds_i=[-y1, -z0]
        crds_j=[-y1, z0]
        crds_k=[-y0, z0]
        crds_l=[-y0, -z0]
        ops.patch("quad", eti_quad_no_conf, num_sub_div_ij, num_sub_div_jk, * crds_i , * crds_j , * crds_k , * crds_l)

        #Definir el acero longitudinal
        mat_tag=4
        #Definir acero longitudinal superior
        num_fiber=self.SUP_INF
        start=[y0, z0]
        end=[y0, -z0]
        ops.layer("straight", mat_tag, num_fiber, area_fiber, *start, *end)

        #Definir acero longitudinal inferior
        num_fiber=self.SUP_INF
        start=[-y0, z0]
        end=[-y0, -z0]
        ops.layer("straight", mat_tag, num_fiber, area_fiber, *start, *end)

        #Definir acero longitudinal derecha
        num_fiber=self.DER_INF
        start=[y0, z0]
        end=[-y0, z0]
        ops.layer("straight", mat_tag, num_fiber, area_fiber, *start, *end)

        #Definir acero longitudinal izquierda
        num_fiber=self.DER_INF
        start=[y0, -z0]
        end=[-y0, -z0]
        ops.layer("straight", mat_tag, num_fiber, area_fiber, *start, *end)

        #Definir agregador de sección 
        eti_agr=2
        ops.section("Aggregator", eti_agr, 3, "Vy", "-section", col_sec)

        #Definir la transformacion de coordenadas
        eti_transf=1
        ops.geomTransf("Linear", eti_transf)

        #Definir los elementos de la columna
        eti_beam=1
        N=10
        ops.beamIntegration("Lobatto", eti_beam, eti_agr, N )
        ele_tag=1
        ele_nodes=[1, 2]
        ops.element("dispBeamColumn", ele_tag, * ele_nodes, eti_transf, eti_beam)

        ele_tag=2
        ele_nodes=[2, 3]
        ops.element("dispBeamColumn", ele_tag, * ele_nodes, eti_transf, eti_beam)

        ele_tag=3
        ele_nodes=[3, 4]
        ops.element("dispBeamColumn", ele_tag, * ele_nodes, eti_transf, eti_beam)

        ele_tag=4
        ele_nodes=[4, 5]
        ops.element("dispBeamColumn", ele_tag, * ele_nodes, eti_transf, eti_beam)

        nodo=5
        dof=2

        #Definir el recorder
        ops.recorder("Node", "-file", self.FILE_INTER_ENS, "-time", "-node", nodo, "-dof", dof, "disp")
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO, "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time", "-node", 1, "-dof", 1, 2, 3, "reaction")
        
        #Definir los patrones de carga
        ops.timeSeries("Constant", 1)
        ops.pattern("Plain", 1, 1)
        ops.load(nodo, -615, 0, 0)

        # Definir los parametros de analisis
        ops.integrator("LoadControl", 0)
        ops.system("SparseGeneral", "-piv")
        ops.test("NormDispIncr", 1.0e-5, 2000)
        ops.constraints("Plain")
        ops.numberer("Plain")
        ops.algorithm("KrylovNewton")
        ops.analysis("Static")

        ok=ops.analyze(1)
        if ok==0:
            print("CARGA AXIAL APLICADA")
        else:
            print("ERROR AL APLICAR LA CARGA AXIAL")
        ops.wipeAnalysis()

        if ok==0:
            # Definir patron de carga
            ops.timeSeries("Linear", 2)
            ops.pattern("Plain", 2, 2)
            ops.load(nodo, 0.0, 1.5, 0.0)
            
            #Definir los nodos de la estructura
            nodos=np.loadtxt(self.data_file)
            

            for i in range(len(nodos)):
                x=float(nodos[i])

                ops.integrator("DisplacementControl", nodo, dof, x)
                ops.analysis("Static")
                ops.analyze(1)


                ops.wipeAnalysis()
        ops.remove("recorders")

        #Personalizar la carga de los datos
        
        serie_total= []        

        if self.calculo == 'REACCION_BASE' :                
            array_mek=np.loadtxt(self.FILE_REAC_BASE)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            


        elif self.calculo == 'DESPLAZAMIENTO_1':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)

        else:            
            datos_1=np.loadtxt(self.FILE_REAC_BASE)  
            datos_2=np.loadtxt(self.FILE_INTER_ENS)                                              
            datos_3=np.loadtxt(self.FILE_DESPLAZAMIENTO)

            serie_total = [[ datos_1, [1,2,3],'REACCION_BASE' ], [ datos_2, [1], 'INTERSECCION' ], [ datos_3, [1,2,3],'DESPLAZAMIENTO' ], ]
            #self.construye_figura(serie_total=serie_total, recoder='*')
        
        nodo=[1, 2]
        dof=[1, 2, 3]

        nodo1=nodo[0]
        nodo2=nodo[1]

        gra_liber1=dof[0]
        gra_liber2=dof[1]
        gra_liber3=dof[2]

        #Resultados

        RESULTADOS = []

        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber1)+" es: "+str(ops.nodeDisp(2, 1))+" m") )        
        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber2)+" es: "+str(ops.nodeDisp(2, 2))+" m") )        
        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber3)+" es: "+str(ops.nodeDisp(2, 3))+" m") )        

        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber1)+" es: "+str(ops.nodeReaction(1, 1))+" N" ))
        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber2)+" es: "+str(ops.nodeReaction(1, 2))+" N") )
        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber3)+" es: "+str(ops.nodeReaction(1, 3))+" N") )

        RESULTADOS.append(str("La reaccción en el elemento: "+ str(nodo1)+ " para el grado de libertad:"+ str(gra_liber1)+" es:"+str(ops.nodeReaction(1, 1))+" N"))

        if serie_total:
            self.construye_figura(serie_total=serie_total, recoder='*', resultados=RESULTADOS)

        self.mostrar_resultados(lista_mensajes=RESULTADOS) 


        #Este el el modelo realizado en el laboratorio                
        self.construye_figura([array_mek, array_mek_lab])


    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
        
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """
                
        titulo = None        
        
        if recoder != '*':

            ventana_principal = plt.figure(0)
            ventana_principal.canvas.set_window_title(recoder)

            for i in serie:
                plt.subplots_adjust(hspace=0.70)
                plt.subplot(2,2,i)
                titulo = "RESPUESTA : GRADO DE LIBERTAD #{}".format(i)            
                plt.title(titulo)                        
                plt.xlabel('Tiempo (seg)')
                plt.ylabel('Desplazamiento (m)')
                print ("Normal :",i)
                plt.plot(array_mek[:,0], array_mek[:,i],label="Resultado")
                plt.legend()                           
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent

            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'Carga_ciclica_01.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)
            

            num_subplots = 0
            
            for elemento in serie_total:
                num_subplots += len(elemento[1]) 
            
            if num_subplots % 2 != 0:
                num_subplots+=1
                        
            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 3
            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax4 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax5 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))
            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 2))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 2))
            
            #Para controlar el subplot de Resultados se visualice
            if COLUMNAS == 4: 
                ax9 = plt.subplot2grid((3,4), (0, 3), rowspan=3)
            

            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
            
                if calculo_var == 'REACCION_BASE':  
                    
                    ax3.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:orange')
                    ax3.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax3.set_xlabel('Fuerza (kN)')
                    ax3.set_ylabel('Desplazamiento (mm)')
                                        
                    ax4.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:orange')
                    ax4.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax4.set_xlabel('Fuerza (kN)')
                    ax4.set_ylabel('Desplazamiento (mm)')
                    
                    ax5.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:orange')
                    ax5.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax5.set_xlabel('Fuerza (kN)')
                    ax5.set_ylabel('Desplazamiento (mm)')

                elif calculo_var == 'INTERSECCION':
                                    
                    ax1.imshow(pil_img) 
                    ax1.set_title(f" ENSAYO POR CARGA CÍCLICA", fontsize=8)
                    ax1.axes.xaxis.set_visible(False)
                    ax1.axes.yaxis.set_visible(False)

                    #ax4.plot(array_mek[0][:,1], array_mek[0][:,0],'tab:orange')
                    ens_opensees=np.loadtxt(self.FILE_INTER_ENS)
                    ens_lab=np.loadtxt(self.data_file_lab)
                    ax2.plot(ens_opensees[:,1], ens_opensees[:,0],'tab:green')
                    ax2.plot(ens_lab[:,0], ens_lab[:,1],'tab:orange')
                    ax2.set_title(f" INTERSECCIÓN DE ENSAYOS", fontsize=8)
                    ax2.set_xlabel('Desplazamiento (m)')
                    ax2.set_ylabel('Fuerza (kN)')


                else:

                    ax6.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:red')
                    ax6.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Fuerza (kN)')
                    ax6.set_ylabel('Desplazamiento (m)')
                    
                    ax7.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:red')
                    ax7.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax7.set_xlabel('Fuerza (kN)')
                    ax7.set_ylabel('Desplazamiento (m)')

                    ax8.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:red')
                    ax8.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Fuerza (kN)')
                    ax8.set_ylabel('Desplazamiento (m)')                
                        
            if COLUMNAS == 4:
                self.mostrar_resultados(lista_mensajes=resultados,ax=ax9)

            fig.suptitle("MODELOS MATEMÁTICOS: ENSAYOS FISICOS")                             
            fig.subplots_adjust(left=0.05, bottom=0.076, right=0.971, top=0.88, wspace=0.562, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    


import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model


class ColumnaPerfil():

    FILE_DESPLAZAMIENTO_NODO2 = "DESPLAZAMIENTO_NODO2.out"          # DESPLAZAMIENTO N2
    FILE_DESPLAZAMIENTO_NODO1 = "DESPLAZAMIENTO_NODO1.out"          # DESPLAZAMIENTO N1
    FILE_REAC_BASE = "REAC_BASE.out"                                # REACCION_BASE      
    FILE_FUERZAS_COL = "FUERZAS_COL.out"                            # FUERZAS COL
    FILE_FUERZAS_COL_N1 = "FUERZAS_COL_N1.out"                      # FUERZAS COL_N1
    FILE_DEFORMACION_COL = "DEFORMACION_COL.out"                    # DEFORMACION_COL
    FILE_FORCE_COLUMNA = "Data/ForceColSecnumIntgrPts.out"          # FORCE_COLUMNA      
    FILE_DEFORMACION_COLUMNA = "Data/DefoColSec$numIntgrPts.out"    # DEFORMACION_COLUMNA

    GRAVEDAD=9.81


    def __init__(self,nombre='BasicBuilder',numero_dimensiones=2, numero_dots=3, altura_columna=12, fuerza_aplicada=1036.21 , resistencia_fc=21000 ,dim_travs_colum_altura=1.5, dim_trav_colum_base=1.5, calculo='*'):
        
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots
        self.nombre = nombre
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.calculo = calculo
        self.altura_columna = altura_columna
        self.fuerza_aplicada = fuerza_aplicada
        self.resistencia_fc = resistencia_fc
        
        ops.wipe()
        #Creación del modelo 
        ops.model(self.nombre, "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

    def construir_modelo(self):

        #Datos de la estructura
        
        peso=self.fuerza_aplicada/self.GRAVEDAD

        #Definir los nodos de la estructura
        ops.node(1, 0, 0)
        ops.node(2, 0, self.altura_columna)

        #Restricciones    
        ops.fixY(0, 1, 1, 1)

        #Definir los materiales
        fy=420000
        E=200000000
        b=0.007
        R0=20
        cr1=0.925
        cr2=0.15
        a1=0.04
        a2=1
        a3=0.04
        a4=1

        params=[R0, cr1, cr2]
        ops.uniaxialMaterial("Steel02", 1, fy, E, b, *params, a1, a2, a3, a4)

        #Definir las etiquetas de secciones
        col_sec=1

        #Creacion de las secciones
        rec=0.02
        y0=self.dim_trav_colum_base/2
        y1=y0-rec
        z0=self.dim_travs_colum_altura/2
        z1=z0-rec
        num_div_z=16
        num_div_y=16
        num_div_rec=2
        ops.section("Fiber", col_sec)

        # Definir los parches para recubriento lateral

        crdsi=[-y0, -z0]
        crdsj=[-y1, -z0]
        crdsk=[-y1, +z0]
        crdsl=[-y0, +z0]

        ops.patch("quad", 1, num_div_rec, (num_div_z+2*num_div_rec), *crdsi, *crdsj, *crdsk, *crdsl)

        crdsi=[y1, -z0]
        crdsj=[y0, -z0]
        crdsk=[y0, +z0]
        crdsl=[y1, +z0]

        ops.patch("quad", 1, num_div_rec, (num_div_z+2*num_div_rec), *crdsi, *crdsj, *crdsk, *crdsl)

        # Definir los parches para recubriento inferior y superior
        crdsi=[-y1, -z0]
        crdsj=[y1, -z0]
        crdsk=[y1, -z1]
        crdsl=[-y1, -z1]

        ops.patch("quad", 1, num_div_y, num_div_rec, *crdsi, *crdsj, *crdsk, *crdsl)

        crdsi=[-y1, -z1]
        crdsj=[y1, -z1]
        crdsk=[y1, +z0]
        crdsl=[-y1, +z0]

        ops.patch("quad", 1, num_div_y, num_div_rec, *crdsi, *crdsj, *crdsk, *crdsl)

        num_subdiv_circ=5
        num_subdiv_rad=5
        centro=[-self.dim_travs_colum_altura/2, -self.dim_trav_colum_base/2]
        rad=[1.58, 3.16]
        ang=[180, 270]
        ops.patch( "circ" , 1, num_subdiv_circ, num_subdiv_rad, * centro, * rad, * ang)

        num_subdiv_circ=5
        num_subdiv_rad=5
        centro=[self.dim_travs_colum_altura/2, -self.dim_trav_colum_base/2]
        rad=[1.58, 3.16]
        ang=[270, 360]
        ops.patch( "circ" , 1, num_subdiv_circ, num_subdiv_rad, * centro, * rad, * ang)

        num_subdiv_circ=5
        num_subdiv_rad=5
        centro=[self.dim_travs_colum_altura/2, self.dim_trav_colum_base/2]
        rad=[1.58, 3.16]
        ang=[0, 90]
        ops.patch( "circ" , 1, num_subdiv_circ, num_subdiv_rad, * centro, * rad, * ang)

        num_subdiv_circ=5
        num_subdiv_rad=5
        centro=[-self.dim_travs_colum_altura/2, self.dim_trav_colum_base/2]
        rad=[1.58, 3.16]
        ang=[90, 180]
        ops.patch( "circ" , 1, num_subdiv_circ, num_subdiv_rad, * centro, * rad, * ang)

        #Definir las etiquetas de transformacion
        col_trans=1

        #Definir los elementos de la columna
        ops.geomTransf("Corotational", col_trans)
        col_intr=1
        N=5
        ops.beamIntegration( 'Lobatto', col_intr, col_sec , N )
        ops.element("forceBeamColumn", 1, 1, 2, col_intr, col_sec, col_trans)

        #Definir los patrones de carga
        ops.timeSeries("Linear", 1)
        ops.pattern("Plain", 1, 1)
        ops.load(2, 0, -self.fuerza_aplicada, 0)

        #Definir los recorders
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO_NODO2, "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO_NODO1, "-time", "-node", 1, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time", "-node", 1, "-dof", 1, 2, 3, "reaction")

        #Crear el analisis
        ops.constraints("Plain")
        ops.numberer("RCM")
        ops.system("UmfPack")
        ops.test("NormDispIncr", 1.0e-5, 10, 0)
        ops.algorithm("Newton")
        num_pasos=10
        ops.integrator("LoadControl", 1/num_pasos)
        ops.analysis("Static")
        ok=ops.analyze(num_pasos)

        if ok==0:
            print("CARGA AXIAL APLICADA")
        else:
            print("ERROR AL APLICAR LA CARGA AXIAL")
            
        ops.loadConst("-time", 0.0)

        if ok==0:
            # Patron de carga axial
            ops.timeSeries("Linear", 2)
            ops.pattern("Plain", 2, 2)
        
            ops.load(2, 0.0, 0.0, 0.0)
                
            #Parametros de analisis estatico
            id_node=2
            d_max_push=0.5
            dx_push=100
            nsteps=d_max_push/dx_push

            ops.constraints("Plain")
            ops.numberer("RCM")
            ops.system("UmfPack")
            ops.test("NormDispIncr", 1.0e-3, 500, 0)
            ops.algorithm("Newton")
            ops.integrator("DisplacementControl", id_node, 1, dx_push)
            ops.analysis("Static")
            
            while ops.nodeDisp(2, 1) <= d_max_push:
                for n in [1, 2, 4, 10]:
                    ops.integrator("DisplacementControl", id_node, 1, nsteps/n)
                    ok=ops.analyze(n)
                    if ok !=0:
                        ops.test("NormDispIncr", 1.0e-5, 5000)
                        ops.algorithm("Newton", "-initial")
                        ok=ops.analyze(n)
                        ops.test("EnergyIncr", 1.0e-6, 100)
                    if ok !=0:
                        ops.test("NormDispIncr", 1.0e-4, 10000)
                        ops.algorithm("Newton", "-initial")
                        ok=ops.analyze(n)
                        ops.test("EnergyIncr", 1.0e-6, 100)
                    if ok !=0:
                        ops.test("NormDispIncr", 1.0e-3, 10000)
                        ops.algorithm("Newton", "-initial")
                        ok=ops.analyze(n)
                        ops.test("EnergyIncr", 1.0e-6, 100)
                    if ok !=0:
                        ops.test("NormDispIncr", 1.0e-2, 10000)
                        ops.algorithm("Newton", "-initial")
                        
                        ok=ops.analyze(n)
                        ops.test("EnergyIncr", 1.0e-6, 100)
                    if ok ==0:
                        break
                if ok !=0:
                    print("EL ANÁLISIS DE DESPLAZAMIENTO FALLÓ: ", ops.nodeDisp(2, 1))
                    break
            if ok==0:
                print("EL ANÁLISIS DE DESPLAZAMIENTO TERMINÓ EXITOSAMENTE")
                
            ops.wipeAnalysis()
        ops.remove("recorders")
                    

        if self.calculo == 'REACCION_BASE' :                
            array_mek=np.loadtxt(self.FILE_REAC_BASE)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            

        elif self.calculo == 'DESPLAZAMIENTO_NODO2':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO2)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)

        elif self.calculo == 'DESPLAZAMIENTO_NODO1':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO1)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)
        
        else:            
            datos_1=np.loadtxt(self.FILE_REAC_BASE)
            #self.construye_figura(array_mek,serie=[1,2,3], recoder='REACCION_BASE')

            datos_2=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO2)
            #self.construye_figura(array_mek,serie=[1,2,3], recoder='DESPLAZAMIENTO_NODO2')

            datos_3=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO1)
            #self.construye_figura(array_mek,serie=[1,2,3], recoder='DESPLAZAMIENTO_NODO1')

            serie_total = [ [ datos_1, [1,2,3],'REACCION_BASE' ], [ datos_2, [1,2,3],'DESPLAZAMIENTO_NODO2' ], [ datos_3, [1,2,3],'DESPLAZAMIENTO_NODO1' ] ]

            if serie_total:
                self.construye_figura(serie_total=serie_total, recoder='*')


        nodo=[1, 2]
        dof=[1, 2, 3]

        nodo1=nodo[0]
        nodo2=nodo[1]

        gra_liber1=dof[0]
        gra_liber2=dof[1]
        gra_liber3=dof[2]

        #Resultados

        RESULTADOS = []

        RESULTADOS.append(str("El desplazamiento en el nodo " + str(nodo2) +" para el grado de libertad " + str(gra_liber1) + " es: " + str(ops.nodeDisp(2, 1))+" m" ))
        RESULTADOS.append(str("El desplazamiento en el nodo " + str(nodo2) +" para el grado de libertad " + str(gra_liber2) + " es: " + str(ops.nodeDisp(2, 2))+" m" ))
        RESULTADOS.append(str("El desplazamiento en el nodo " + str(nodo2) +" para el grado de libertad " + str(gra_liber3) + " es: " + str(ops.nodeDisp(2, 3))+" rad/s" ))

        RESULTADOS.append(str("La reaccción en el nodo "+ str(nodo1) +" para el grado de libertad " + str(gra_liber1) + " es: " + str(ops.nodeReaction(1, 1))+" kN" ))
        RESULTADOS.append(str("La reaccción en el nodo "+ str(nodo1) +" para el grado de libertad " + str(gra_liber2) + " es: " + str(ops.nodeReaction(1, 2))+" kN" ))
        RESULTADOS.append(str("La reaccción en el nodo "+ str(nodo1) +" para el grado de libertad " + str(gra_liber3) + " es: " + str(ops.nodeReaction(1, 3))+" rad/s" ))




        
    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], RESULTADOS=[]):

        """Construye una figura basado en un array de datos, una serie de gráficos        
        """

        if recoder != '*':

            nombre_figura = "Gráfico # "
            titulo = None

            for i in serie:

                if recoder != None:
                    titulo = recoder+' : '+nombre_figura+str(i)
                    plt.figure(titulo)
                else:
                    titulo = nombre_figura + str(i)
                    plt.figure(titulo)
                
                plt.plot(array_mek[:,0], array_mek[:,i], "-.")
                
                plt.ylabel('Carga aplicada (N)')
                plt.xlabel('Desplazamiento horizontal (m)')
                plt.show()
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent

            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'Columna_con_perfil_metalico.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)
            

            num_subplots = 0
            
            for elemento in serie_total:
                num_subplots += len(elemento[1]) 
            
            if num_subplots % 2 != 0:
                num_subplots+=1
                        
            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 4
            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax1.imshow(pil_img) 
            ax1.axes.xaxis.set_visible(False)
            ax1.axes.yaxis.set_visible(False)

            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax2.axes.xaxis.set_visible(False)
            ax2.axes.yaxis.set_visible(False)
            ax2.spines['top'].set_visible(False)
            ax2.spines['right'].set_visible(False)
            ax2.spines['bottom'].set_visible(False)
            ax2.spines['left'].set_visible(False)  

            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax4 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax5 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))
            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 2))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 2))

            ax9 = plt.subplot2grid((FILAS, COLUMNAS), (0, 3))
            ax10 = plt.subplot2grid((FILAS, COLUMNAS), (1, 3))
            ax11 = plt.subplot2grid((FILAS, COLUMNAS), (2, 3))
                                                
            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
            
                if calculo_var == 'REACCION_BASE':  
                    
                    ax3.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:orange')
                    ax3.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax3.set_xlabel('Desplazamiento Horizontal (m)')
                    ax3.set_ylabel('Carga Aplicada  (N)')
                                        
                    ax4.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:orange')
                    ax4.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax4.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax4.set_ylabel('Carga Aplicada  (N)')
                    
                    ax5.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:orange')
                    ax5.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax5.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax5.set_ylabel('Carga Aplicada  (N)')

                elif calculo_var == 'DESPLAZAMIENTO_NODO2':
                                        
                    ax6.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:red')
                    ax6.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax6.set_ylabel('Carga Aplicada  (N)')
                    
                    ax7.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:red')
                    ax7.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax7.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax7.set_ylabel('Carga Aplicada  (N)')

                    ax8.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:red')
                    ax8.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax8.set_ylabel('Carga Aplicada  (N)')      

                elif calculo_var == 'DESPLAZAMIENTO_NODO1':

                    ax9.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:blue')
                    ax9.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax9.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax9.set_ylabel('Carga Aplicada  (N)')
                    
                    ax10.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:blue')
                    ax10.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax10.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax10.set_ylabel('Carga Aplicada  (N)')

                    ax11.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:blue')
                    ax11.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax11.set_xlabel('Desplazamiento Horizontal (mm)')
                    ax11.set_ylabel('Carga Aplicada  (N)')     

                else:
                    pass         
                        
            fig.suptitle("MODELOS MATEMÁTICOS: COLUMNA MIXTA HORMIGÓN-ACERO CON CARGA PUNTUAL")                             
            fig.subplots_adjust(left=0.05, bottom=0.121, right=0.902, top=0.88, wspace=0.562, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    def mostrar_resultados(self, lista_mensajes=[],ax=None):
                
        fig, ax1 = plt.subplots(1, 1, constrained_layout=True, sharey=True)       
        ax1.set_xlabel('Desplazamiento Horizontal (mm)')
        ax1.set_ylabel('Carga Aplicada  (N)')   
                        
        resultado = ''
        
        for index, cadena in enumerate(lista_mensajes):           
            resultado += cadena+'\n'
                        
        ax1.axes.xaxis.set_visible(False)
        ax1.axes.yaxis.set_visible(False)
        ax1.spines['top'].set_visible(False)
        ax1.spines['right'].set_visible(False)
        ax1.spines['bottom'].set_visible(False)
        ax1.spines['left'].set_visible(False)
        ax1.text(0.038, 0.75, resultado , horizontalalignment='left', verticalalignment='center_baseline', transform=ax1.transAxes, bbox={'facecolor': 'orange', 'alpha': 0.25, 'pad': 5}, fontsize=9)        
        fig.suptitle('\n\nResultados', fontsize=16)

        plt.show()

    


import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model


class ExcitacionSinuidal():

    FILE_DESPLAZAMIENTO_NODO2 = "DESPLAZAMIENTO_NODO2.out"          # DESPLAZAMIENTO N2
    FILE_DESPLAZAMIENTO_NODO1 = "DESPLAZAMIENTO_NODO1.out"          # DESPLAZAMIENTO N1
    FILE_REAC_BASE = "REAC_BASE.out"                                # REACCION_BASE      
    FILE_FUERZAS_COL_SEC = "Fuerzas_Col_sec.out"                    # FUERZAS COL SEC
    FILE_FUERZAS_COL = "FUERZAS_COL.out"                            # FUERZAS COL

    FILE_DEFORMACION_COL = "DEFORMACION_COL.out"                    # DEFORMACION_COL
    FILE_FORCE_COLUMNA = "Data/ForceColSecnumIntgrPts.out"          # FORCE_COLUMNA      
    FILE_DEFORMACION_COLUMNA = "Data/DefoColSec$numIntgrPts.out"    # DEFORMACION_COLUMNA
    FILE_ROTACION_PLASTICA = "PlasticRotation.out"                  # ROTACION PLASTICA     
    FILE_ELEMENTO_AB = "ElementoAB.out"                             # ELEMENTO AB
    GRAVEDAD=9.81


    def __init__(self,nombre='BasicBuilder',numero_dimensiones=2, numero_dots=3, altura_columna=4, fuerza_aplicada=15000 , resistencia_fc=21000 ,dim_travs_colum_altura=1.5, dim_trav_colum_base=1.5, calculo='*'):
        
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots
        self.nombre = nombre
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.calculo = calculo
        self.altura_columna = altura_columna
        self.fuerza_aplicada = fuerza_aplicada
        self.resistencia_fc = resistencia_fc
        

        ops.wipe()
        #Creación del modelo 
        ops.model(self.nombre, "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

    
    def construir_modelo(self):
        
        #Datos de la estructura
        H_col=1.5 #float(input("Ingrese las dimensiones tranversal de la columna, altura: "))
        B_col=1.5 #float(input("Ingrese las dimensiones tranversal de la columna, base: "))
        h=12 #float(input("Ingrese la altura de la columna: "))
        fuer_apli=8000 #float(input("Ingrese la fuerza aplicada a la columna: "))
        fc=21000 #float(input("Ingrese la resistencia a la compresión: "))
        pes_esp_hor=23.56
        peso_ele=pes_esp_hor*H_col*B_col*h
        peso_estru=peso_ele+fuer_apli
        g=9.81
        masa=peso_estru/g
        masa_1=masa-323.896
        Area=H_col*B_col
        I=(1/12)*B_col*H_col**3
        E=20000000

        #Definir los nodos de la estructura

        ops.node(1, 0, 0)
        ops.node(2, 0, h)

        #Restricciones    
        ops.fix(1, 1, 1, 1)


        #Definir la masa en la cabeza de la columna
        ops.mass(2, masa, 1e-8, 1e-8)

        # resistencia nominal a la compresión del hormigón
        fc1=fc
        Ec=E

        # Definir la sección de columna
        My_col=130000
        Phi_Y_col=0.65e-4
        EI_col_crack=My_col/Phi_Y_col
        b=0.01

        ops.uniaxialMaterial("Steel01", 2, My_col, EI_col_crack, b)
        ops.uniaxialMaterial("Elastic", 3, Ec*Area)
        ops.section("Aggregator", 1, 3, "P", 2, "Mz")

        #Definir la transformacion de coordenadas
        ops.geomTransf("PDelta", 1)
        num_intgr_pts=5
        ops.element("nonlinearBeamColumn", 1, 1, 2, num_intgr_pts, 1, 1)

        # Propiedades del hormigón y acero
        fc1=fc
        Ec=E
        fy=420000
        fyh=420000
        Es=200000000
        shr=0.003

        #Columnas
        dia_bar_long=16
        diametro=dia_bar_long/1000
        num_arr_abajo=8
        num_izq_der=4
        dia_est=10
        estribo=dia_est/1000
        num_ram_Y=2
        num_ram_X=2
        recubrimiento=4
        rec=recubrimiento/100
        sep_est=0.1

        # Definir el modelo de mander
        eco=0.002
        epss=0.004
        esm=0.1
        sp=sep_est*estribo
        rs=(num_ram_X*estribo**2*(np.pi/4)*(B_col-2*rec)+num_ram_Y*estribo**2*(np.pi/4)*(H_col-2*rec))/((B_col-2*rec)*(H_col-2*rec)*sep_est)
        area_var=diametro**2*(np.pi/4)
        area_con=(B_col-2*rec)*(H_col-2*rec)
        rcc=area_var*(num_arr_abajo+num_izq_der)/area_con
        wi=0.2
        ke=((1-wi**2/(6*area_con))*(1-sep_est/(2*B_col))*(1-sep_est/(2*H_col)))/(1-rcc)
        fpl=1/(2*ke*rs*fyh)
        fpcc=(-1.254+2.254*(1+7.94*fpl/fc1)**0.5-2*fpl/fc1)*fc1
        ecc=eco*(1+5*(fpcc/(fc1-1)))
        Esec=fpcc/ecc
        r=Ec/(Ec-Esec)
        ecu=1.5*(0.004+1.4*rs*fyh*(esm/fpcc))
        x=ecu/ecc
        fcu=fpcc*x*r/(r-1+x**r)

        #Definir los materiales de la columna
        #Concreto incofinado
        ops.uniaxialMaterial("Concrete01", 4, -fc1, -eco, 0, -epss)

        #Acero longitudinal
        ops.uniaxialMaterial("Steel01", 5, fy, Es, shr)

        #Concreto cofinado
        ops.uniaxialMaterial("Concrete01", 6, -fpcc, -ecc, -fcu, -ecu)
        ops.uniaxialMaterial("Elastic", 7, Ec)

        #Creacion de las secciones
        y0=B_col/2
        y1=y0-rec
        z0=H_col/2
        z1=z0-rec
        num_div_Z=16
        num_div_Y=16
        num_div_rec=2
        ops.section("Fiber", 10)

        # Definir los parches para recubriento lateral

        crdsI=[-y0, -z0]
        crdsJ=[-y1, -z0]
        crdsK=[-y1, +z0]
        crdsL=[-y0, +z0]

        ops.patch("quad", 4, num_div_rec, (num_div_Z+2*num_div_rec), *crdsI, *crdsJ, *crdsK, *crdsL)

        crdsI=[y1, -z0]
        crdsJ=[y0, -z0]
        crdsK=[y0, +z0]
        crdsL=[y1, +z0]

        ops.patch("quad", 4, num_div_rec, (num_div_Z+2*num_div_rec), *crdsI, *crdsJ, *crdsK, *crdsL)

        # Definir los parches para recubriento inferior y superior
        crdsI=[-y1, -z0]
        crdsJ=[y1, -z0]
        crdsK=[y1, -z1]
        crdsL=[-y1, -z1]

        ops.patch("quad", 4, num_div_Y, num_div_rec, *crdsI, *crdsJ, *crdsK, *crdsL)

        crdsI=[-y1, -z1]
        crdsJ=[y1, -z1]
        crdsK=[y1, +z0]
        crdsL=[-y1, +z0]

        ops.patch("quad", 4, num_div_Y, num_div_rec, *crdsI, *crdsJ, *crdsK, *crdsL)

        # Definir los parches para hormigon confinado
        crdsI=[-y1, -z1]
        crdsJ=[y1, -z1]
        crdsK=[y1, z1]
        crdsL=[-y1, z1]

        ops.patch("quad", 6, num_div_Y, num_div_rec, *crdsI, *crdsJ, *crdsK, *crdsL)

        # Acero superior e inferior
        ops.fiber(0, z1, ((num_arr_abajo/2)*area_var), 5)
        ops.fiber(0, -z1, ((num_arr_abajo/2)*area_var), 5)

        # Acero superior e inferior
        ops.fiber(y1, 0, ((num_izq_der/2)*area_var), 5)
        ops.fiber(y1, 0, ((num_izq_der/2)*area_var), 5)

        ops.section("Aggregator", 2, 7, "Vy", "-section", 10)

        #Definir los recorders
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO_NODO2, "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO_NODO1, "-time", "-node", 1, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time", "-node", 1, "-dof", 1, 2, 3, "reaction")

        ops.recorder("Element", "-file", self.FILE_FUERZAS_COL, "-time", "-ele", 1, "globalForce")        
        ops.recorder("Element", "-file", self.FILE_FUERZAS_COL_SEC, "-time", "-ele", 1, "section", 10, "force")

        ops.recorder("Element", "-file", "Defor_col_sec.out", "-time", "-ele", 1, "section", 10, "deformation")
        ops.recorder("Element", "-file", "Fuerzas_Col_sec_num_intgr_pts.out", "-time", "-ele", 1, "section", num_intgr_pts, "force")
        ops.recorder("Element", "-file", "Defor_Col_sec_num_intgr_pts.out", "-time", "-ele", 1, "section", num_intgr_pts, "force")

        #Definir las actuantes sobre la columna
        ops.timeSeries("Linear", 1)
        ops.pattern("Plain", 1, 1)
        ops.eleLoad("-ele", 1, "-type", "-beamUniform", -masa)

        #Parametros de analisis estatico
        ops.constraints("Transformation")
        ops.numberer("RCM")
        ops.system("BandGeneral")
        ops.test("EnergyIncr", 1e-8, 5, 0)
        ops.algorithm("ModifiedNewton")
        ops.integrator("Newmark", 0.5, 0.25)
        ops.analysis("Transient")

        #Analisis modal
        Num_modos=1 #int(input("Ingrese el numero de modos a calcular: "))
        OmegaSq=ops.eigen("-fullGenLapack", Num_modos)
        OmegaSq=np.array(OmegaSq)
        Omega=OmegaSq**0.5
        Periodo=(2*np.pi)/Omega

        for i in range(Num_modos):
            print("Modo", i+1, "T= ", Periodo[i], "s")
            
        # Definir la excitación de la onda sinusoidal
        Tmax=100
        dt=0.01
        g=9.81
        Ampl_ond_sinusoidal=0.5*g
        T_onda_sinusoidal=0.35
        Dur_onda_sinusoidal=5
        Dir=1
        Npuntos=1000

        Omega_onda_sinusoidal=(2*np.pi/T_onda_sinusoidal)
        Vel0=Ampl_ond_sinusoidal*(-1)/Omega_onda_sinusoidal
        ops.timeSeries("Sine", 0, Dur_onda_sinusoidal, T_onda_sinusoidal, "-factor", Ampl_ond_sinusoidal)
        ops.pattern("UniformExcitation", 8, Dir, "-accel", 0, "-vel0", Vel0)

        #Parametros de analisis estatico

        ops.constraints("Plain")
        ops.numberer("RCM")
        ops.system("UmfPack")
        ops.test("NormDispIncr", 1.0e-8, 10)
        ops.algorithm("Newton")
        ops.integrator("Newmark", 0.5, 0.25)
        ops.analysis("Transient")
        ops.analyze(Npuntos, dt)

        #ops.remove("recorders")

        if self.calculo == 'REACCION_BASE' :                
            array_mek=np.loadtxt(self.FILE_REAC_BASE)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            
        elif self.calculo == 'DESPLAZAMIENTO_NODO2':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO2)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)
        elif self.calculo == 'DESPLAZAMIENTO_NODO1':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO1)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)
        
        elif self.calculo == 'FUERZAS_COL':            
            array_mek=np.loadtxt(self.FILE_FUERZAS_COL)           
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)

        elif self.calculo == 'FUERZAS_COL_N1':            
            array_mek=np.loadtxt(self.FILE_FUERZAS_COL_N1)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)
        
        elif self.calculo == 'DEFORMACION_COL':            
            array_mek=np.loadtxt(self.FILE_DEFORMACION_COL)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)

        elif self.calculo == 'FORCE_COLUMNA':            
            array_mek=np.loadtxt(self.FILE_FORCE_COLUMNA)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)

        elif self.calculo == 'DEFORMACION_COLUMNA':            
            array_mek=np.loadtxt(self.FILE_DEFORMACION_COLUMNA)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)

        else:            
            
            #datos_1=np.loadtxt(self.FILE_REAC_BASE)
            #self.construye_figura(array_mek,serie=[1,2,3], recoder='REACCION_BASE')

            datos_2=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO2)
            #self.construye_figura(array_mek,serie=[1,2,3], recoder='DESPLAZAMIENTO_NODO2')

            #datos_3=np.loadtxt(self.FILE_DESPLAZAMIENTO_NODO1)
            #self.construye_figura(array_mek,serie=[1,2,3], recoder='DESPLAZAMIENTO_NODO1')

            """array_mek=np.loadtxt(self.FILE_FUERZAS_COL)
            self.construye_figura(array_mek,serie=[1], recoder='FUERZAS_COL')

            array_mek=np.loadtxt(self.FILE_FUERZAS_COL_N1)
            self.construye_figura(array_mek,serie=[1], recoder='FUERZAS_COL_N1')

            array_mek=np.loadtxt(self.FILE_DEFORMACION_COL)
            self.construye_figura(array_mek,serie=[1], recoder='DEFORMACION_COL')

            array_mek=np.loadtxt(self.FILE_FORCE_COLUMNA)
            self.construye_figura(array_mek,serie=[1], recoder='FORCE_COLUMNA')

            array_mek=np.loadtxt(self.FILE_DEFORMACION_COLUMNA)
            self.construye_figura(array_mek,serie=[1], recoder='DEFORMACION_COLUMNA')"""

            serie_total = [ [ datos_2, [1,2,3],'DESPLAZAMIENTO_NODO2' ] ]

            if serie_total:
                self.construye_figura(serie_total=serie_total, recoder='*')
            

    def concrete_section(self, core_tag, cover_tag, B, H, cover, nfcore, nfcover):
        ops.patch("rect", core_tag, nfcore, 1, -H/2+cover, -B/2+cover, H/2+cover, B/2+cover)
        ops.patch("rect", cover_tag, nfcover, 1, -H/2, -B/2, -H/2+cover, B/2)
        ops.patch("rect", cover_tag, nfcover, 1, H/2-cover, -B/2, H/2, B/2)
        ops.patch("rect", cover_tag, nfcore, 1, -H/2+cover, -B/2, H/2-cover, B/2+cover)
        ops.patch("rect", cover_tag, nfcore, 1, -H/2+cover, B/2-cover, H/2-cover, B/2)

    
    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
    
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """
        
        if recoder != '*':

            nombre_figura = "Gráfico # "
            titulo = None

            for i in serie:

                if recoder != None:
                    titulo = recoder+' : '+nombre_figura+str(i)
                    plt.figure(titulo)
                else:
                    titulo = nombre_figura + str(i)
                    plt.figure(titulo)
                
                plt.plot(array_mek[:,0], array_mek[:,i])
                
                plt.ylabel('Desplazamiento horizontal (m)')
                plt.xlabel('Tiempo (s)')
                plt.show()
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent

            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'columna.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)
            

            num_subplots = 0
            
            for elemento in serie_total:
                num_subplots += len(elemento[1]) 
            
            if num_subplots % 2 != 0:
                num_subplots+=1
                        
            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 3
            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax1.imshow(pil_img) 
            ax1.axes.xaxis.set_visible(False)
            ax1.axes.yaxis.set_visible(False)

            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax2.axes.xaxis.set_visible(False)
            ax2.axes.yaxis.set_visible(False)
            ax2.spines['top'].set_visible(False)
            ax2.spines['right'].set_visible(False)
            ax2.spines['bottom'].set_visible(False)
            ax2.spines['left'].set_visible(False)  

            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))

                                                
            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]

                if calculo_var == 'DESPLAZAMIENTO_NODO2':
                                        
                    ax6.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:red')
                    ax6.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Tiempo (s)')
                    ax6.set_ylabel('Desplazamiento H. (m)')
                    
                    ax7.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:red')
                    ax7.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax7.set_xlabel('Tiempo (s)')
                    ax7.set_ylabel('Desplazamiento H. (m)')

                    ax8.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:red')
                    ax8.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Tiempo (s)')
                    ax8.set_ylabel('Desplazamiento H. (m)')      


                        
            fig.suptitle("MODELOS MATEMÁTICOS: EXCITACION SINUIDAL")                             
            fig.subplots_adjust(left=0.305, bottom=0.076, right=0.92, top=0.88, wspace=0.430, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    def mostrar_resultados(self, lista_mensajes=[],ax=None):
                
        fig, ax1 = plt.subplots(1, 1, constrained_layout=True, sharey=True)       
        ax1.set_xlabel('Tiempo (s)')
        ax1.set_ylabel('Desplazamiento H. (m)')   
                        
        resultado = ''
        
        for index, cadena in enumerate(lista_mensajes):           
            resultado += cadena+'\n'
                        
        ax1.axes.xaxis.set_visible(False)
        ax1.axes.yaxis.set_visible(False)
        ax1.spines['top'].set_visible(False)
        ax1.spines['right'].set_visible(False)
        ax1.spines['bottom'].set_visible(False)
        ax1.spines['left'].set_visible(False)
        ax1.text(0.038, 0.75, resultado , horizontalalignment='left', verticalalignment='center_baseline', transform=ax1.transAxes, bbox={'facecolor': 'orange', 'alpha': 0.25, 'pad': 5}, fontsize=9)        
        fig.suptitle('\n\nResultados', fontsize=16)

        plt.show()

    
        


import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model


class AmortiguadorViscoso():

    
    #Variables globales : CONSTANTES

    PERIODO=0.7
    KD=25
    CD=20.7452
    AD=0.35
    H=3
    PESO=10000

    
    #RECODERS
    FILE_DESPLAZAMIENTO_N1 = "Despla_nodo_1.out"    # DESPLAZAMIENTO_N1
    FILE_DESPLAZAMIENTO_N2 = "Despla_nodo_2.out"    # DESPLAZAMIENTO_N2
    FILE_REAC_BASE = "Reacc_Base.out"               # REACCION_BASE      
    FILE_FUERZAS_COL = "Fuerzas_Col.out"            # FUERZAS_COLUMNA
    FILE_FUERZAS_COL_SEC = "Fuerzas_Col_sec.out"    # FUERZAS_COLUMNA SEC
    FILE_DEFORMACION_COL_SEC = "Defor_col_sec.out"  # DEFORMACION



    def __init__(self, nombre='BasicBuilder', numero_dimensiones=2, numero_dots=3,altura_columna=0.6, base_columna=0.6, numero_modos=2,data_file="amortiguador.txt",calculo='*'): 
        
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots
        self.altura_columna = altura_columna
        self.base_columna = base_columna
        self.nombre = nombre
        self.numero_modos = numero_modos
        self.data_file = data_file
        self.calculo = calculo

        #Creación del modelo 
        ops.wipe()
        ops.model(self.nombre, "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

    def construir_modelo(self):

        #Definir los nodos de la estructura
        ops.node(1, 0, 0)
        ops.node(2, 0, self.H)

        #Restricciones    
        ops.fix(1, 1, 1, 1)
        ops.fix(2, 0, 1, 0)

        #Definir la masa en la cabeza de la columna
        ops.mass(2, self.PESO, 1e-8, 1e-8)

        #Definir las propiedades de la columna
        #K=(np.pi/self.PERIODO)**2
        E=20000000
        Ic=(self.H**3/24*E)

        area = self.altura_columna*self.base_columna

        #Definir el material viscoso
        ops.uniaxialMaterial("ViscousDamper", 1, self.KD, self.CD, self.AD)

        #Definir la transformacion de coordenadas
        transformacion_coordenadas=1
        ops.geomTransf("Linear", transformacion_coordenadas)

        #Se definen los elementos.
        ops.element("elasticBeamColumn", 1, 1, 2, area, E, Ic, transformacion_coordenadas)

        # Amortiguador
        #element twoNodeLink $eleTag $iNode $jNode -mat $matTags -dir $dirs
        ops.element("twoNodeLink", 2, 1, 2, "-mat", 1, "-dir", 1)

        #Definir los recorders
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO_N2, "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO_N1, "-time", "-node", 1, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time", "-node", 1, "-dof", 1, 2, 3, "reaction")
        ops.recorder("Element", "-file", self.FILE_FUERZAS_COL, "-time", "-ele", 1, "localForce")
        ops.recorder("Element", "-file", self.FILE_FUERZAS_COL_SEC, "-time", "-ele", 1, "deformations")
        ops.recorder("Element", "-file", self.FILE_DEFORMACION_COL_SEC, "-time", "-ele", 1, "-dof", 1, "force")

        #Analisis modal
        #numero_modos=20 #int(input("Ingrese el numero de modos a calcular: "))
        
        omega_sq = ops.eigen("-fullGenLapack", self.numero_modos)
        omega_sq = np.array(omega_sq)

        omega = omega_sq**0.5
        periodo =(2*np.pi)/omega

        for i in range(self.numero_modos):
            print("Modo", i+1, "T= ", periodo[i], "s")

        #Definir análisis dinamico
        factor=0.001
        dt=0.001
        n_puntos=15000
        
        var_dir=1
        ops.timeSeries("Path", 2, "-dt", dt, "-filePath", self.data_file, "-factor", factor)
        ops.pattern("UniformExcitation", 2, var_dir, "-accel", 2)

        #Parametros de analisis dinamico
        ops.constraints("Transformation")
        ops.numberer("RCM")
        ops.system("UmfPack")
        ops.test("EnergyIncr", 1.0e-8, 10)
        ops.algorithm("KrylovNewton")
        ops.integrator("Newmark", 0.5, 0.25)
        ops.analysis("Transient")
        ops.analyze(n_puntos, dt)
        ops.remove("recorders")


        if self.calculo == 'DESPLAZAMIENTO_N2' :    
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO_N2)            
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            
        elif self.calculo == 'DESPLAZAMIENTO_N1':
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO_N1)                
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            
        elif self.calculo == 'REACCION_BASE':
            array_mek=np.loadtxt(self.FILE_REAC_BASE)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            
        elif self.calculo == 'FUERZAS_COLUMNA':
            array_mek=np.loadtxt(self.FILE_FUERZAS_COL)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)                            
        elif self.calculo == 'FUERZAS_COLUMNA_SEC':
            array_mek=np.loadtxt(self.FILE_FUERZAS_COL_SEC)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)                            
        elif self.calculo == 'DEFORMACION_COL_SEC':
            array_mek=np.loadtxt(self.FILE_DEFORMACION_COL_SEC)            
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)                            
        else:
            datos_1=np.loadtxt(self.FILE_DESPLAZAMIENTO_N2)                                    
            datos_2=np.loadtxt(self.FILE_DESPLAZAMIENTO_N1)                            
            datos_3=np.loadtxt(self.FILE_REAC_BASE)            
            datos_4=np.loadtxt(self.FILE_FUERZAS_COL)            
            serie_total = [[ datos_1, [1,2,3],'DESPLAZAMIENTO_N2' ], [ datos_2, [1,2,3],'DESPLAZAMIENTO_N1' ], [ datos_3, [1,2,3],'REACCION_BASE' ], [ datos_4, [1],'FUERZAS_COLUMNA' ] ]     

            if serie_total:
                self.construye_figura(serie_total=serie_total, recoder=self.calculo)
                        

    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
            
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """

        if recoder != '*':

            nombre_figura = "Gráfico # "
            titulo = None

            for i in serie:

                if recoder != None:
                    titulo = recoder+' : '+nombre_figura+str(i)
                    plt.figure(titulo)
                else:
                    titulo = nombre_figura + str(i)
                    plt.figure(titulo)
                
                plt.plot(array_mek[:,0], array_mek[:,i])
                
                plt.ylabel('Desplazamiento Horizontal (m)')
                plt.xlabel('Tiempo (s)')                       
                plt.show()

        else:
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent
            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'columna_con_carga_ciclica.jpeg')
            pil_img = Image.open(IMAGEN_COLUMNA)
                                                
            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 4

            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax1.imshow(pil_img) 
            ax1.axes.xaxis.set_visible(False)
            ax1.axes.yaxis.set_visible(False)    

            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))

            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax4 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax5 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))

            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 2))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 2))

            ax9 = plt.subplot2grid((FILAS, COLUMNAS), (0, 3))
            ax10 = plt.subplot2grid((FILAS, COLUMNAS), (1, 3))
            ax11 = plt.subplot2grid((FILAS, COLUMNAS), (2, 3))
            
            
            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
            
                if calculo_var == 'DESPLAZAMIENTO_N2':  
                    
                    ax3.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:red')
                    ax3.set_title(f" {calculo_var}\nGRAFICO : # {serie[0]}", fontsize=8)
                    ax3.set_xlabel('Tiempo (seg)')
                    ax3.set_ylabel('Desplazamiento H. (m)')
                                        
                    ax4.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:red')
                    ax4.set_title(f" {calculo_var}\nGRAFICO : # {serie[1]}", fontsize=8)
                    ax4.set_xlabel('Tiempo (seg)')
                    ax4.set_ylabel('Desplazamiento H. (m)')
                    
                    ax5.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:red')
                    ax5.set_title(f" {calculo_var}\nGRAFICO : # {serie[2]}", fontsize=8)
                    ax5.set_xlabel('Tiempo (seg)')
                    ax5.set_ylabel('Desplazamiento H. (m)')

                elif calculo_var == 'DESPLAZAMIENTO_N1':
                                        
                    ax6.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:orange')
                    ax6.set_title(f" {calculo_var}\nGRAFICO : # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Tiempo (seg)')
                    ax6.set_ylabel('Desplazamiento H. (m)')
                                        
                    ax7.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:orange')
                    ax7.set_title(f" {calculo_var}\nGRAFICO : # {serie[1]}", fontsize=8)
                    ax7.set_xlabel('Tiempo (seg)')
                    ax7.set_ylabel('Desplazamiento H. (m)')
                    
                    ax8.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:orange')
                    ax8.set_title(f" {calculo_var}\nGRAFICO : # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Tiempo (seg)')
                    ax8.set_ylabel('Desplazamiento H. (m)')

                elif calculo_var == 'REACCION_BASE':
                                        
                    ax9.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:blue')
                    ax9.set_title(f" {calculo_var}\nGRAFICO : # {serie[0]}", fontsize=8)
                    ax9.set_xlabel('Tiempo (seg)')
                    ax9.set_ylabel('Desplazamiento H. (m)')
                                        
                    ax10.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:blue')
                    ax10.set_title(f" {calculo_var}\nGRAFICO : # {serie[1]}", fontsize=8)
                    ax10.set_xlabel('Tiempo (seg)')
                    ax10.set_ylabel('Desplazamiento H. (m)')
                    
                    ax11.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:blue')
                    ax11.set_title(f" {calculo_var}\nGRAFICO : # {serie[2]}", fontsize=8)
                    ax11.set_xlabel('Tiempo (seg)')
                    ax11.set_ylabel('Desplazamiento H. (m)')

                else :

                    ax2.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:green')  
                    ax2.set_title(f" {calculo_var}\nGRAFICO : # {serie[0]}", fontsize=8)
                    ax2.set_xlabel('Tiempo (seg)')
                    ax2.set_ylabel('Desplazamiento H. (m)')
                
            fig.suptitle("MODELOS MATEMÁTICOS: AMORTIGUADOR VISCOSO ")                             
            fig.subplots_adjust(left=0.067, bottom=0.107, right=0.971, top=0.88, wspace=0.562, hspace=1)

            #Para fijar un titulo a la ventana principal
            mng = plt.get_current_fig_manager()
            mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
            mng.resize(*mng.window.maxsize())                            
            plt.show() 


from setuptools import setup, find_packages

setup(
    install_requires=["openseespy","numpy","matplotlib"],
    name='utpl-modelos-columnas',
    version='4.0',
    description='Modelos de Columnas- Tesis de Grado',
    author='Ronald Valencia',
    author_email='ravalencia2@utpl.edu.ec',
    url='https://github.com/ravalenciaochoa/utpl',
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: GNU General Public License (GPL)",
    ]
)



import openseespy.opensees as ops
import numpy as np
import matplotlib.pyplot as plt
from openseespy.postprocessing.Get_Rendering import plot_model


class Acelerograma():

    #Variables de Clase
    area = None
    numero_pasos=10
    E=20000000
    G=9.81

    #RECODERS
    FILE_DESPLAZAMIENTO = "DESPLAZAMIENTO.out"  # DESPLAZAMIENTO
    FILE_REAC_BASE = "REAC_BASE.out"            # REACCION_BASE
    FILE_SOLIC_COL = "SOLIC_COL.out"            # SOLICITACION

    def __init__(self,nombre='BasicBuilder',numero_dimensiones=2, numero_dots=3, data_file='datos.txt', altura_columna=12, masa=1500, dim_travs_colum_altura=1.5, dim_trav_colum_base=1.5, calculo='*'):

        self.nombre = nombre
        self.numero_dimensiones = numero_dimensiones
        self.numero_dots = numero_dots
        self.data_file = data_file

        #Datos de la Estructura
        self.altura_columna = altura_columna
        self.masa = masa
        self.dim_travs_colum_altura = dim_travs_colum_altura
        self.dim_trav_colum_base = dim_trav_colum_base
        self.calculo = calculo

        ops.wipe()
        ops.model(self.nombre, "-ndm", self.numero_dimensiones, "-ndf", self.numero_dots)

    def construir_modelo(self):

        """Construye el modelo con los valores proporcionados por el usuario en el constructor de clase
        """

        area=self.dim_travs_colum_altura*self.dim_trav_colum_base
        I=(1/12)*self.dim_trav_colum_base*self.dim_travs_colum_altura**3

        #Definir los nodos de la estructura
        ops.node(1, 0, 0)
        ops.node(2, 0, self.altura_columna)

        #Definir la masa de la estructura
        ops.mass(2, self.masa, 0, 0)

        #Definir las restricciones
        ops.fix(1, 1, 1, 1)

        #Definir la transformacion geometrica
        ops.geomTransf("Linear", 1)

        #Definir elemento de la columna
        ops.element("elasticBeamColumn", 1, 1, 2, area, self.E, I, 1)

        #Definir los recorders
        ops.recorder("Node", "-file", self.FILE_DESPLAZAMIENTO, "-time", "-node", 2, "-dof", 1, 2, 3, "disp")
        ops.recorder("Node", "-file", self.FILE_REAC_BASE, "-time", "-node", 1, "-dof", 1, 2, 3, "reaction")
        ops.recorder("Element", "-file", self.FILE_SOLIC_COL, "-time", "-ele", 1, "force", 1)

        #Definir las actuantes sobre la columna
        ops.timeSeries("Linear", 1)
        ops.pattern("Plain", 1, 1)
        ops.load(2, 0, (-1*self.masa*self.G), 0)

        #Parametros de analisis estatico

        ops.constraints("Plain")
        ops.numberer("RCM")
        ops.system("BandGeneral")
        ops.test("NormDispIncr", 1.0e-6, 10)
        ops.algorithm("Newton")        
        ops.integrator("LoadControl", 1/self.numero_pasos)
        ops.analysis("Static")
        ops.analyze(10)

        #Definir análisis dinamico
        ops.loadConst("-time", 0)
        numero_nodos=1
        factor=0.01
        dt=0.01
        numero_puntos=10000

        valor_dir=1
        ops.timeSeries("Path", 2, "-dt", dt, "-filePath", self.data_file, "-factor", factor)
        ops.pattern("UniformExcitation", 2, valor_dir, "-accel", 2)

        #Parametros de analisis estatico

        ops.constraints("Plain")
        ops.numberer("RCM")
        ops.system("UmfPack")
        ops.test("NormDispIncr", 1.0e-8, 10)
        ops.algorithm("Newton")
        ops.integrator("Newmark", 0.5, 0.25)
        ops.analysis("Transient")
        ops.analyze(numero_puntos, dt)

        #Definir el amortiguamiento
        omegasq=ops.eigen("-fullGenLapack", 1)
        omegasq=np.array(omegasq)
        omega=omegasq**0.5
        periodo=(2*np.pi)/omega
        ops.rayleigh(0, 0, 0, 0.01)

        for i in range(numero_nodos):
            print("Modo", i+1, "T= ", periodo[i], "s")
            
        ops.remove("recorders")
            
        #Personalizar la carga de los datos
        # 
        serie_total= []        

        if self.calculo == 'REACCION_BASE' :                
            array_mek=np.loadtxt(self.FILE_REAC_BASE)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)                            

        elif self.calculo == 'SOLICITACION':            
            array_mek=np.loadtxt(self.FILE_SOLIC_COL)
            self.construye_figura(array_mek,serie=[1], recoder=self.calculo)

        elif self.calculo == 'DESPLAZAMIENTO':            
            array_mek=np.loadtxt(self.FILE_DESPLAZAMIENTO)
            self.construye_figura(array_mek,serie=[1,2,3], recoder=self.calculo)

        else:            
            datos_1=np.loadtxt(self.FILE_REAC_BASE)                        
            datos_2=np.loadtxt(self.FILE_SOLIC_COL)                        
            datos_3=np.loadtxt(self.FILE_DESPLAZAMIENTO)

            serie_total = [[ datos_1, [1,2,3],'REACCION_BASE' ], [ datos_2, [1],    'SOLICITACION' ], [ datos_3, [1,2,3],'DESPLAZAMIENTO' ] ]
            #self.construye_figura(serie_total=serie_total, recoder='*')
        
        nodo=[1, 2]
        dof=[1, 2, 3]

        nodo1=nodo[0]
        nodo2=nodo[1]

        gra_liber1=dof[0]
        gra_liber2=dof[1]
        gra_liber3=dof[2]

        #Resultados

        RESULTADOS = []

        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber1)+" es: "+str(ops.nodeDisp(2, 1))+" m") )        
        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber2)+" es: "+str(ops.nodeDisp(2, 2))+" m") )        
        RESULTADOS.append(str("El desplazamiento en el nodo: " + str(nodo2)+ " para el grado de libertad: "+str(gra_liber3)+" es: "+str(ops.nodeDisp(2, 3))+" m") )        

        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber1)+" es: "+str(ops.nodeReaction(1, 1))+" N" ))
        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber2)+" es: "+str(ops.nodeReaction(1, 2))+" N") )
        RESULTADOS.append(str("La reaccción en el nodo: " + str(nodo1)+ " para el grado de libertad: "+str(gra_liber3)+" es: "+str(ops.nodeReaction(1, 3))+" N") )

        RESULTADOS.append(str("La reaccción en el elemento: "+ str(nodo1)+ " para el grado de libertad:"+ str(gra_liber1)+" es:"+str(ops.nodeReaction(1, 1))+" N"))

        if serie_total:
            self.construye_figura(serie_total=serie_total, recoder='*', resultados=RESULTADOS)

        self.mostrar_resultados(lista_mensajes=RESULTADOS)

        #plot_model("nodes","elements") 




    def construye_figura(self, array_mek=None, serie=[], recoder=None, serie_total=[], resultados=[]):
        
        """Construye una figura basado en un array de datos, una serie de gráficos        
        """
                
        titulo = None        
        
        if recoder != '*':

            ventana_principal = plt.figure(0)
            ventana_principal.canvas.set_window_title(recoder)

            for i in serie:
                plt.subplots_adjust(hspace=0.70)
                plt.subplot(2,2,i)
                titulo = "RESPUESTA : GRADO DE LIBERTAD #{}".format(i)            
                plt.title(titulo)                        
                plt.xlabel('Tiempo (seg)')
                plt.ylabel('Desplazamiento (m)')
                print ("Normal :",i)
                plt.plot(array_mek[:,0], array_mek[:,i],label="Resultado")
                plt.legend()                           
        else :
            
            from PIL import Image
            import matplotlib.image as mpimg
            import os
            from pathlib import Path
            
            BASE_DIR = Path(__file__).resolve().parent.parent

            IMAGEN_COLUMNA = os.path.join(os.path.join(BASE_DIR,'modelos'), 'Columna_con_carga_ciclica.jpeg')

            pil_img = Image.open(IMAGEN_COLUMNA)
            

            num_subplots = 0
            
            for elemento in serie_total:
                num_subplots += len(elemento[1]) 
            
            if num_subplots % 2 != 0:
                num_subplots+=1
                        
            fig = plt.figure()


            FILAS = 3
            COLUMNAS = 3
            ax1 = plt.subplot2grid((FILAS, COLUMNAS), (0, 0), rowspan=2)
            ax2 = plt.subplot2grid((FILAS, COLUMNAS), (2, 0))
            ax3 = plt.subplot2grid((FILAS, COLUMNAS), (0, 1))
            ax4 = plt.subplot2grid((FILAS, COLUMNAS), (1, 1))
            ax5 = plt.subplot2grid((FILAS, COLUMNAS), (2, 1))
            ax6 = plt.subplot2grid((FILAS, COLUMNAS), (0, 2))
            ax7 = plt.subplot2grid((FILAS, COLUMNAS), (1, 2))
            ax8 = plt.subplot2grid((FILAS, COLUMNAS), (2, 2))
            
            #Para controlar el subplot de Resultados se visualice
            if COLUMNAS == 4: 
                ax9 = plt.subplot2grid((3,4), (0, 3), rowspan=3)
            

            for columna, elemento in enumerate(serie_total):                
                
                array_mek = elemento[0]
                serie = elemento[1]
                calculo_var = elemento[2]
            
                if calculo_var == 'REACCION_BASE':  
                    
                    ax3.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:orange')
                    ax3.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax3.set_xlabel('Tiempo (seg)')
                    ax3.set_ylabel('Desplazamiento (m)')
                                        
                    ax4.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:orange')
                    ax4.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax4.set_xlabel('Tiempo (seg)')
                    ax4.set_ylabel('Desplazamiento (m)')
                    
                    ax5.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:orange')
                    ax5.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax5.set_xlabel('Tiempo (seg)')
                    ax5.set_ylabel('Desplazamiento (m)')

                elif calculo_var == 'SOLICITACION':
                                        
                    ax1.imshow(pil_img) 
                    ax1.axes.xaxis.set_visible(False)
                    ax1.axes.yaxis.set_visible(False)
                    
                    ax2.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:green')  
                    ax2.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax2.set_xlabel('Tiempo (seg)')
                    ax2.set_ylabel('Desplazamiento (m)')

                else:

                    ax6.plot(array_mek[:,0], array_mek[:,serie[0]],'tab:red')
                    ax6.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[0]}", fontsize=8)
                    ax6.set_xlabel('Tiempo (seg)')
                    ax6.set_ylabel('Desplazamiento (m)')
                    
                    ax7.plot(array_mek[:,0], array_mek[:,serie[1]],'tab:red')
                    ax7.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[1]}", fontsize=8)
                    ax7.set_xlabel('Tiempo (seg)')
                    ax7.set_ylabel('Desplazamiento (m)')

                    ax8.plot(array_mek[:,0], array_mek[:,serie[2]],'tab:red')
                    ax8.set_title(f" {calculo_var}\nGRADO DE LIBERTAD # {serie[2]}", fontsize=8)
                    ax8.set_xlabel('Tiempo (seg)')
                    ax8.set_ylabel('Desplazamiento (m)')                
                        
            if COLUMNAS == 4:
                self.mostrar_resultados(lista_mensajes=resultados,ax=ax9)

            fig.suptitle("MODELOS MATEMÁTICOS: COLUMNA DE HORMIGÓN ARMADO CON MOVIMIENTO DE BASE")                             
            fig.subplots_adjust(left=0.05, bottom=0.076, right=0.971, top=0.88, wspace=0.562, hspace=1)
    
        mng = plt.get_current_fig_manager()
        mng.set_window_title("MODELOS MATEMÁTICOS UTPL")
        mng.resize(*mng.window.maxsize())                
        plt.show() 

    def mostrar_resultados(self, lista_mensajes=[],ax=None):
                
        fig, ax1 = plt.subplots(1, 1, constrained_layout=True, sharey=True)       
        ax1.set_xlabel('Tiempo (seg)')
        ax1.set_ylabel('Desplazamiento (m)')   
                        
        resultado = ''
        
        for index, cadena in enumerate(lista_mensajes):           
            resultado += cadena+'\n'
                        
        ax1.axes.xaxis.set_visible(False)
        ax1.axes.yaxis.set_visible(False)
        ax1.spines['top'].set_visible(False)
        ax1.spines['right'].set_visible(False)
        ax1.spines['bottom'].set_visible(False)
        ax1.spines['left'].set_visible(False)
        ax1.text(0.038, 0.75, resultado , horizontalalignment='left', verticalalignment='center_baseline', transform=ax1.transAxes, bbox={'facecolor': 'orange', 'alpha': 0.25, 'pad': 5}, fontsize=9)        
        fig.suptitle('\n\nResultados', fontsize=16)

        plt.show()

        
        

        

        

